@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import sys
-if sys.version_info < (2,5):
-    print "Sorry, requires Python 2.5 or higher."
+if sys.version_info < (2, 5):
+    print "Sorry, requires Python 2.5, 2.6 or 2.7."
     sys.exit(1)
 
 import logging
@@ -78,7 +78,7 @@ import sabnzbd.config as config
 import sabnzbd.cfg
 import sabnzbd.downloader
 from sabnzbd.encoding import unicoder, latin1
-from sabnzbd.utils import osx
+import sabnzbd.growler as growler
 
 from threading import Thread
 
@@ -231,7 +231,7 @@ def print_help():
     print "  -t  --templates <templ>  Template directory [*]"
     print "  -2  --template2 <templ>  Secondary template dir [*]"
     print
-    print "  -l  --logging <0..2>     Set logging level (0= least, 2= most) [*]"
+    print "  -l  --logging <0..2>     Set logging level (-1=off, 0= least, 2= most) [*]"
     print "  -w  --weblogging <0..2>  Set cherrypy logging (0= off, 1= on, 2= file-only) [*]"
     print
     print "  -b  --browser <0..1>     Auto browser launch (0= off, 1= on) [*]"
@@ -253,12 +253,13 @@ def print_help():
     print "      --log-all            Log all article handling (for developers)"
     print "      --console            Force console logging for OSX app"
     print "      --new                Run a new instance of SABnzbd"
+    print "      --no_ipv6            Do not listen on IPv6 address [::1]"
 
 def print_version():
     print """
 %s-%s
 
-Copyright (C) 2008-2011, The SABnzbd-Team <team@sabnzbd.org>
+Copyright (C) 2008-2012, The SABnzbd-Team <team@sabnzbd.org>
 SABnzbd comes with ABSOLUTELY NO WARRANTY.
 This is free software, and you are welcome to redistribute it
 under certain conditions. It is licensed under the
@@ -321,7 +322,8 @@ def Web_Template(key, defweb, wdir):
             wdir = ''
     if not wdir:
         wdir = defweb
-    key.set(wdir)
+    if key:
+        key.set(wdir)
     if not wdir:
         # No default value defined, accept empty path
         return ''
@@ -331,6 +333,10 @@ def Web_Template(key, defweb, wdir):
     logging.info("Web dir is %s", full_dir)
 
     if not os.path.exists(full_main):
+        # Temporarily fix that allows missing Config
+        if defweb == DEF_STDCONFIG:
+            return ''
+        # end temp fix
         logging.warning(Ta('Cannot find web template: %s, trying standard template'), full_main)
         full_dir = real_path(sabnzbd.DIR_INTERFACES, DEF_STDINTF)
         full_main = real_path(full_dir, DEF_MAIN_TMPL)
@@ -360,12 +366,14 @@ def fix_webname(name):
         xname = name.title()
     else:
         xname = ''
-    if xname in ('Default',):
+    if xname in ('Default', ):
         return 'Classic'
     elif xname in ('Classic', 'Plush', 'Mobile'):
         return xname
     elif xname in ('Smpl', 'Wizard'):
         return name.lower()
+    elif xname in ('Config',):
+        return 'Plush'
     else:
         return name
 
@@ -494,21 +502,49 @@ def print_modules():
 
 #------------------------------------------------------------------------------
 def all_localhosts():
-    """ Return all unique values of localhost """
-    return ['127.0.0.1']
+    """ Return all unique values of localhost in order of preference """
+    ips = ['127.0.0.1']
+    try:
+        # Check whether IPv6 is available and enabled
+        info = socket.getaddrinfo('::1', None)
+        af, socktype, proto, canonname, sa = info[0]
+        s = socket.socket(af, socktype, proto)
+        s.close()
+    except socket.error:
+        return ips
+    try:
+        info = socket.getaddrinfo('localhost', None)
+    except:
+        # localhost does not resolve
+        return ips
     ips = []
-    for item in socket.getaddrinfo('localhost', None):
+    for item in info:
         item = item[4][0]
         if item not in ips:
             ips.append(item)
     return ips
 
 
+def check_resolve(host):
+    """ Return True if 'host' resolves
+    """
+    try:
+        info = socket.getaddrinfo(host, None)
+    except:
+        # Does not resolve
+        return False
+    return True
+
 #------------------------------------------------------------------------------
 def get_webhost(cherryhost, cherryport, https_port):
     """ Determine the webhost address and port,
         return (host, port, browserhost)
     """
+    if cherryhost == '0.0.0.0' and not check_resolve('127.0.0.1'):
+        cherryhost = ''
+    elif cherryhost == '::' and not check_resolve('::1'):
+        cherryhost = ''
+
     if cherryhost is None:
         cherryhost = sabnzbd.cfg.cherryhost()
     else:
@@ -521,9 +557,18 @@ def get_webhost(cherryhost, cherryport, https_port):
     try:
         info = socket.getaddrinfo(socket.gethostname(), None)
     except:
-        # Hostname does not resolve, use 0.0.0.0
-        cherryhost = '0.0.0.0'
-        info = socket.getaddrinfo(localhost, None)
+        # Hostname does not resolve
+        try:
+            # Valid user defined name?
+            info = socket.getaddrinfo(cherryhost, None)
+        except:
+            if cherryhost not in ('localhost', '127.0.0.1', '::1'):
+                cherryhost = '0.0.0.0'
+            try:
+                info = socket.getaddrinfo(localhost, None)
+            except:
+                info = socket.getaddrinfo('127.0.0.1', None)
+                localhost = '127.0.0.1'
     for item in info:
         ip = str(item[4][0])
         if ip.startswith('169.254.'):
@@ -617,7 +662,7 @@ def get_webhost(cherryhost, cherryport, https_port):
         # if the https port was specified, assume they want HTTPS enabling also
         sabnzbd.cfg.enable_https.set(True)
 
-    if cherryport == https_port:
+    if cherryport == https_port and sabnzbd.cfg.enable_https():
         sabnzbd.cfg.enable_https.set(False)
         # Should have a translated message, but that's not available yet
         #logging.error(Ta('HTTP and HTTPS ports cannot be the same'))
@@ -629,13 +674,14 @@ def get_webhost(cherryhost, cherryport, https_port):
 def attach_server(host, port, cert=None, key=None):
     """ Define and attach server, optionally HTTPS
     """
-    http_server = _cpwsgi_server.CPWSGIServer()
-    http_server.bind_addr = (host, port)
-    if cert and key:
-        http_server.ssl_certificate = cert
-        http_server.ssl_private_key = key
-    adapter = _cpserver.ServerAdapter(cherrypy.engine, http_server, http_server.bind_addr)
-    adapter.subscribe()
+    if not (sabnzbd.cfg.no_ipv6() and '::1' in host):
+        http_server = _cpwsgi_server.CPWSGIServer()
+        http_server.bind_addr = (host, port)
+        if cert and key:
+            http_server.ssl_certificate = cert
+            http_server.ssl_private_key = key
+        adapter = _cpserver.ServerAdapter(cherrypy.engine, http_server, http_server.bind_addr)
+        adapter.subscribe()
 
 
 def is_sabnzbd_running(url):
@@ -732,7 +778,7 @@ def evaluate_inipath(path):
     inipath = os.path.join(path, DEF_INI_FILE)
     if os.path.isdir(path):
         return inipath
-    elif os.path.isfile(path):
+    elif os.path.isfile(path) or os.path.isfile(path + '.bak'):
         return path
     else:
         dirpart, name = os.path.split(path)
@@ -795,7 +841,7 @@ def commandline_handler(frozen=True):
     try:
         opts, args = getopt.getopt(info, "phdvncw:l:s:f:t:b:2:",
                                    ['pause', 'help', 'daemon', 'nobrowser', 'clean', 'logging=',
-                                    'weblogging=', 'server=', 'templates',
+                                    'weblogging=', 'server=', 'templates', 'no_ipv6',
                                     'template2', 'browser=', 'config-file=', 'force',
                                     'version', 'https=', 'autorestarted', 'repair', 'repair-all',
                                     'log-all', 'no-login', 'pid=', 'new', 'sessions', 'console',
@@ -873,6 +919,7 @@ def main():
     new_instance = False
     force_sessions = False
     osx_console = False
+    no_ipv6 = False
 
     service, sab_opts, serv_opts, upload_nzbs = commandline_handler()
 
@@ -921,8 +968,8 @@ def main():
             try:
                 logging_level = int(arg)
             except:
-                logging_level = -1
-            if logging_level < 0 or logging_level > 2:
+                logging_level = -2
+            if logging_level < -1 or logging_level > 2:
                 print_help()
                 exit_sab(1)
         elif opt in ('-v', '--version'):
@@ -959,6 +1006,8 @@ def main():
         elif opt in ('--console',):
             re_argv.append(opt)
             osx_console = True
+        elif opt in ('--no_ipv6',):
+            no_ipv6 = True
 
     sabnzbd.MY_FULLNAME = os.path.normpath(os.path.abspath(sabnzbd.MY_FULLNAME))
     sabnzbd.MY_NAME = os.path.basename(sabnzbd.MY_FULLNAME)
@@ -980,7 +1029,7 @@ def main():
     if noConsoleLoggingOSX:
         consoleLogging = 1
 
-    LOGLEVELS = (logging.WARNING, logging.INFO, logging.DEBUG)
+    LOGLEVELS = (logging.FATAL, logging.WARNING, logging.INFO, logging.DEBUG)
 
     # Setup primary logging to prevent default console logging
     gui_log = guiHandler(MAX_WARNINGS)
@@ -1012,7 +1061,7 @@ def main():
         GetProfileInfo(vista_plus)
         # Find out where INI file is
         inifile = os.path.abspath(sabnzbd.DIR_PROG + '/' + DEF_INI_FILE)
-        if not os.path.exists(inifile):
+        if not os.path.exists(inifile) and not os.path.exists(inifile + '.bak'):
             inifile = os.path.abspath(sabnzbd.DIR_LCLDATA + '/' + DEF_INI_FILE)
             if sabnzbd.DARWIN:
                 copy_old_files(sabnzbd.DIR_LCLDATA)
@@ -1024,7 +1073,7 @@ def main():
     # All system data dirs are relative to the place we found the INI file
     sabnzbd.DIR_LCLDATA = os.path.dirname(inifile)
 
-    if not os.path.exists(inifile) and not os.path.exists(sabnzbd.DIR_LCLDATA):
+    if not os.path.exists(inifile) and not os.path.exists(inifile + '.bak') and not os.path.exists(sabnzbd.DIR_LCLDATA):
         try:
             os.makedirs(sabnzbd.DIR_LCLDATA)
         except IOError:
@@ -1041,6 +1090,9 @@ def main():
     # Set root folders for HTTPS server file paths
     sabnzbd.cfg.set_root_folders2()
 
+    if no_ipv6:
+        sabnzbd.cfg.no_ipv6.set(True)
+
     # Determine web host address
     cherryhost, cherryport, browserhost, https_port = get_webhost(cherryhost, cherryport, https_port)
     enable_https = sabnzbd.cfg.enable_https()
@@ -1159,13 +1211,11 @@ def main():
             sabnzbd.cfg.log_backups())
 
         format = '%(asctime)s::%(levelname)s::[%(module)s:%(lineno)d] %(message)s'
-        if sabnzbd.WIN32:
-            format += '\r'
         rollover_log.setFormatter(logging.Formatter(format))
         rollover_log.addFilter(FilterCP3())
         sabnzbd.LOGHANDLER = rollover_log
         logger.addHandler(rollover_log)
-        logger.setLevel(LOGLEVELS[logging_level])
+        logger.setLevel(LOGLEVELS[logging_level+1])
 
     except IOError:
         print "Error:"
@@ -1193,7 +1243,7 @@ def main():
             if consoleLogging:
                 console = logging.StreamHandler()
                 console.addFilter(FilterCP3())
-                console.setLevel(LOGLEVELS[logging_level])
+                console.setLevel(LOGLEVELS[logging_level+1])
                 console.setFormatter(logging.Formatter(format))
                 logger.addHandler(console)
             if noConsoleLoggingOSX:
@@ -1254,12 +1304,14 @@ def main():
 
     web_dir  = Web_Template(sabnzbd.cfg.web_dir,  DEF_STDINTF,  fix_webname(web_dir))
     web_dir2 = Web_Template(sabnzbd.cfg.web_dir2, '', fix_webname(web_dir2))
+    web_dirc = Web_Template(None,  DEF_STDCONFIG, '')
 
     wizard_dir = os.path.join(sabnzbd.DIR_INTERFACES, 'wizard')
     #sabnzbd.lang.install_language(os.path.join(wizard_dir, DEF_INT_LANGUAGE), sabnzbd.cfg.language(), 'wizard')
 
     sabnzbd.WEB_DIR  = web_dir
     sabnzbd.WEB_DIR2 = web_dir2
+    sabnzbd.WEB_DIRC = web_dirc
     sabnzbd.WIZARD_DIR = wizard_dir
 
     sabnzbd.WEB_COLOR = CheckColor(sabnzbd.cfg.web_color(),  web_dir)
@@ -1273,6 +1325,10 @@ def main():
     # Save the INI file
     config.save_config(force=True)
 
+    if sabnzbd.WIN32 and sabnzbd.cfg.win_menu() and not sabnzbd.DAEMON:
+        import sabnzbd.sabtray
+        sabnzbd.WINTRAY = sabnzbd.sabtray.SABTrayThread()
+
     print_modules()
 
     cherrylogtoscreen = False
@@ -1312,8 +1368,8 @@ def main():
         hosts[1] = '::1'
 
     # The Windows binary requires numeric localhost as primary address
-    if multilocal and cherryhost == 'localhost' and hosts[1] == '127.0.0.1':
-        cherryhost = '::1'
+    if multilocal and cherryhost == 'localhost':
+        cherryhost = hosts[0]
 
     if enable_https:
         if https_port:
@@ -1357,17 +1413,21 @@ def main():
                             'engine.reexec_retry' : 100,
                             'tools.encode.on' : True,
                             'tools.gzip.on' : True,
+                            'tools.gzip.mime_types' : ['text/html', 'text/plain', 'text/javascript', 'text/css', 'application/x-javascript'],
                             'tools.sessions.on' : bool(sessions),
                             'tools.sessions.storage_type' : 'file',
                             'tools.sessions.storage_path' : sessions,
                             'tools.sessions.timeout' : 60,
                             'request.show_tracebacks': True,
                             'checker.check_localhost' : bool(consoleLogging),
-                            'error_page.401': sabnzbd.panic.error_page_401
+                            'error_page.401': sabnzbd.panic.error_page_401,
+                            'error_page.404': sabnzbd.panic.error_page_404
                             })
 
 
     static = {'tools.staticdir.on': True, 'tools.staticdir.dir': os.path.join(web_dir, 'static')}
+    if web_dirc:
+        staticcfg = {'tools.staticdir.on': True, 'tools.staticdir.dir': os.path.join(web_dirc, 'staticcfg')}
     wizard_static = {'tools.staticdir.on': True, 'tools.staticdir.dir': os.path.join(wizard_dir, 'static')}
 
     appconfig = {'/sabnzbd/api' : {'tools.basic_auth.on' : False},
@@ -1382,6 +1442,9 @@ def main():
                  '/sabnzbd/wizard/static': wizard_static,
                  '/wizard/static': wizard_static
                  }
+    if web_dirc:
+        appconfig['/sabnzbd/staticcfg'] = staticcfg
+        appconfig['/staticcfg'] = staticcfg
 
     if web_dir2:
         static2 = {'tools.staticdir.on': True, 'tools.staticdir.dir': os.path.join(web_dir2, 'static')}
@@ -1392,8 +1455,11 @@ def main():
         appconfig['/m/static'] = static2
         appconfig['/sabnzbd/m/wizard/static'] = wizard_static
         appconfig['/m/wizard/static'] = wizard_static
+        if web_dirc:
+            appconfig['/sabnzbd/m/staticcfg'] = staticcfg
+            appconfig['/m/staticcfg'] = staticcfg
 
-    login_page = sabnzbd.interface.MainPage(web_dir, '/', web_dir2, '/m/', first=2)
+    login_page = sabnzbd.interface.MainPage(web_dir, '/', web_dir2, '/m/', web_dirc, first=2)
     cherrypy.tree.mount(login_page, '/', config=appconfig)
 
     # Set authentication for CherryPy
@@ -1428,9 +1494,9 @@ def main():
 
     if enable_https:
         browser_url = "https://%s:%s/sabnzbd" % (browserhost, cherryport)
-        cherrypy.wsgiserver.REDIRECT_URL = browser_url
     else:
         browser_url = "http://%s:%s/sabnzbd" % (browserhost, cherryport)
+    cherrypy.wsgiserver.REDIRECT_URL = browser_url
 
     sabnzbd.BROWSER_URL = browser_url
     if not autorestarted:
@@ -1438,12 +1504,12 @@ def main():
         if sabnzbd.FOUNDATION:
             import sabnzbd.osxmenu
             sabnzbd.osxmenu.notify("SAB_Launched", None)
-        osx.sendGrowlMsg('SABnzbd %s' % (sabnzbd.__version__),"http://%s:%s/sabnzbd" % (browserhost, cherryport),osx.NOTIFICATION['startup'])
+        growler.send_notification('SABnzbd %s' % (sabnzbd.__version__),
+                             "http://%s:%s/sabnzbd" % (browserhost, cherryport), 'startup')
         # Now's the time to check for a new version
         check_latest_version()
     autorestarted = False
 
-
     mail = None
     if sabnzbd.WIN32:
         if enable_https:
@@ -1500,7 +1566,7 @@ def main():
         # Check for loglevel changes
         if LOG_FLAG:
             LOG_FLAG = False
-            level = LOGLEVELS[sabnzbd.cfg.log_level()]
+            level = LOGLEVELS[sabnzbd.cfg.log_level()+1]
             logger.setLevel(level)
             if consoleLogging:
                 console.setLevel(level)
@@ -1566,6 +1632,9 @@ def main():
 
     config.save_config()
 
+    if sabnzbd.WINTRAY:
+        sabnzbd.WINTRAY.terminate = True
+
     if sabnzbd.WIN_SERVICE and mail:
         mail.send('stop')
     if sabnzbd.WIN32:
@@ -1578,8 +1647,7 @@ def main():
     if getattr(sys, 'frozen', None) == 'macosx_app':
         AppHelper.stopEventLoop()
     else:
-        if sabnzbd.DARWIN:
-            osx.sendGrowlMsg('SABnzbd',T('SABnzbd shutdown finished'),osx.NOTIFICATION['startup'])
+        growler.send_notification('SABnzbd',T('SABnzbd shutdown finished'), 'startup')
         os._exit(0)
 
 
@@ -1702,7 +1770,14 @@ def HandleCommandLine(allow_service=True):
 #
 if __name__ == '__main__':
 
-    sabnzbd.CMDLINE = ', '.join(['"%s"' % latin1(p) for p in sys.argv])
+    args = []
+    for txt in sys.argv:
+        if ' ' in txt:
+            txt = '"%s"' % latin1(txt)
+        else:
+            txt = latin1(txt)
+        args.append(txt)
+    sabnzbd.CMDLINE = ' '.join(args)
 
     if sabnzbd.WIN32:
         if not HandleCommandLine(allow_service=not hasattr(sys, "frozen")):
@@ -1738,4 +1813,4 @@ if __name__ == '__main__':
             main()
 
     else:
-        main()
+        main()
\ No newline at end of file
