@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -59,6 +59,10 @@ elif os.name == 'posix':
             pass
         if platform.machine() == 'i386':
             DARWIN_INTEL = True
+if DARWIN:
+    DARWIN_ML = [int(n) for n in platform.mac_ver()[0].split('.')] >= [10, 8]
+else:
+    DARWIN_ML = False
 
 #------------------------------------------------------------------------
 
@@ -119,15 +123,17 @@ CMDLINE = ''  # Rendering of original command line arguments
 
 WEB_DIR = None
 WEB_DIR2 = None
+WEB_DIRC = None
 WIZARD_DIR = None
 WEB_COLOR = None
 WEB_COLOR2 = None
 SABSTOP = False
 RESTART_REQ = False
-OSX_ICON = 1
 PAUSED_ALL = False
 OLD_QUEUE = False
 SCHED_RESTART = False # Set when restarted through scheduler
+WINTRAY = None # Thread for the Windows SysTray icon
+WEBUI_READY = False
 
 __INITIALIZED__ = False
 __SHUTTING_DOWN__ = False
@@ -151,7 +157,7 @@ LANG_MAP = {
 # Signal Handler                                                               #
 ################################################################################
 def sig_handler(signum = None, frame = None):
-    global SABSTOP
+    global SABSTOP, WINTRAY
     if sabnzbd.WIN32 and type(signum) != type(None) and DAEMON and signum==5:
         # Ignore the "logoff" event when running as a Win32 daemon
         return True
@@ -163,6 +169,9 @@ def sig_handler(signum = None, frame = None):
         if sabnzbd.WIN32:
             from util.apireg import del_connection_info
             del_connection_info()
+            if sabnzbd.WINTRAY:
+                sabnzbd.WINTRAY.terminate = True
+                time.sleep(0.5)
         else:
             pid_file()
         SABSTOP = True
@@ -186,7 +195,7 @@ def initialize(pause_downloader = False, clean_up = False, evalSched=False, repa
            LOGFILE, WEBLOGFILE, LOGHANDLER, GUIHANDLER, AMBI_LOCALHOST, WAITEXIT, \
            DAEMON, MY_NAME, MY_FULLNAME, NEW_VERSION, \
            DIR_HOME, DIR_APPDATA, DIR_LCLDATA, DIR_PROG , DIR_INTERFACES, \
-           DARWIN, RESTART_REQ, OSX_ICON, OLD_QUEUE
+           DARWIN, RESTART_REQ, OLD_QUEUE
 
     if __INITIALIZED__:
         return False
@@ -196,6 +205,9 @@ def initialize(pause_downloader = False, clean_up = False, evalSched=False, repa
     ### Set global database connection for Web-UI threads
     cherrypy.engine.subscribe('start_thread', connect_db)
 
+    ### Paused?
+    pause_downloader = pause_downloader or cfg.start_paused()
+
     ### Clean-up, if requested
     if clean_up:
         # Old cache folder
@@ -204,6 +216,9 @@ def initialize(pause_downloader = False, clean_up = False, evalSched=False, repa
         # New admin folder
         misc.remove_all(cfg.admin_dir.get_path(), '*.sab')
 
+    ### Set access rights for "incomplete" base folder
+    misc.set_permissions(cfg.download_dir.get_path(), recursive=False)
+
     ### If dirscan_dir cannot be created, set a proper value anyway.
     ### Maybe it's a network path that's temporarily missing.
     path = cfg.dirscan_dir.get_path()
@@ -227,17 +242,31 @@ def initialize(pause_downloader = False, clean_up = False, evalSched=False, repa
     cfg.bandwidth_limit.callback(guard_speedlimit)
     cfg.top_only.callback(guard_top_only)
     cfg.pause_on_post_processing.callback(guard_pause_on_pp)
+    cfg.growl_server.callback(sabnzbd.growler.change_value)
+    cfg.growl_password.callback(sabnzbd.growler.change_value)
+    cfg.quota_size.callback(guard_quota_size)
+    cfg.quota_day.callback(guard_quota_dp)
+    cfg.quota_period.callback(guard_quota_dp)
+    cfg.fsys_type.callback(guard_fsys_type)
+    cfg.language.callback(sabnzbd.growler.reset_growl)
+
+    ### Set Posix filesystem encoding
+    sabnzbd.encoding.change_fsys(cfg.fsys_type())
 
     ### Set cache limit
+    if (sabnzbd.WIN32 or sabnzbd.DARWIN) and not cfg.cache_limit():
+        cfg.cache_limit.set('200M')
     ArticleCache.do.new_limit(cfg.cache_limit.get_int())
 
+    check_incomplete_vs_complete()
+
     ### Handle language upgrade from 0.5.x to 0.6.x
     cfg.language.set(LANG_MAP.get(cfg.language(), cfg.language()))
 
     ### Set language files
     lang.set_locale_info('SABnzbd', DIR_LANGUAGE)
     lang.set_language(cfg.language())
-    sabnzbd.api.cache_skin_trans()
+    sabnzbd.api.clear_trans_cache()
 
     ### Check for old queue (when a new queue is not present)
     if not os.path.exists(os.path.join(cfg.cache_dir.get_path(), QUEUE_FILE_NAME)):
@@ -264,7 +293,7 @@ def initialize(pause_downloader = False, clean_up = False, evalSched=False, repa
     Bookmarks()
     rss.init()
 
-    BPSMeter.do.read()
+    paused = BPSMeter.do.read()
 
     PostProcessor()
 
@@ -273,7 +302,7 @@ def initialize(pause_downloader = False, clean_up = False, evalSched=False, repa
 
     Assembler()
 
-    Downloader(pause_downloader)
+    Downloader(pause_downloader or paused)
 
     DirScanner()
 
@@ -316,6 +345,11 @@ def start():
         logging.debug('Starting urlgrabber')
         URLGrabber.do.start()
 
+        if DARWIN_ML:
+            logging.debug('Starting OSX keep-awake service')
+            osxmenu.OsxAwake()
+            osxmenu.OsxAwake.do.start()
+
 
 @synchronized(INIT_LOCK)
 def halt():
@@ -375,6 +409,9 @@ def halt():
         except:
             logging.error('Fatal error at saving state', exc_info=True)
 
+        if DARWIN_ML:
+            osxmenu.OsxAwake.do.stop()
+
 
         # The Scheduler cannot be stopped when the stop was scheduled.
         # Since all warm-restarts have been removed, it's not longer
@@ -417,6 +454,18 @@ def guard_pause_on_pp():
     else:
         Downloader.do.resume_from_postproc()
 
+def guard_quota_size():
+    """ Callback for change of quota_size """
+    BPSMeter.do.change_quota()
+
+def guard_quota_dp():
+    """ Callback for change of quota_day or quota_period """
+    scheduler.restart(force=True)
+
+def guard_fsys_type():
+    """ Callback for change of file system naming type """
+    sabnzbd.encoding.change_fsys(cfg.fsys_type())
+
 def add_msgid(msgid, pp=None, script=None, cat=None, priority=None, nzbname=None):
     """ Add NZB based on newzbin report number, attributes optional
     """
@@ -571,10 +620,9 @@ def add_nzbfile(nzbfile, pp=None, script=None, cat=None, priority=NORMAL_PRIORIT
             logging.info("Traceback: ", exc_info = True)
 
     if ext.lower() in ('.zip', '.rar'):
-        ProcessArchiveFile(filename, path, pp, script, cat, priority=priority)
+        return ProcessArchiveFile(filename, path, pp, script, cat, priority=priority)
     else:
-        ProcessSingleFile(filename, path, pp, script, cat, priority=priority, nzbname=nzbname, keep=keep, reuse=reuse)
-
+        return ProcessSingleFile(filename, path, pp, script, cat, priority=priority, nzbname=nzbname, keep=keep, reuse=reuse)
 
 ################################################################################
 ## Unsynchronized methods                                                     ##
@@ -718,11 +766,14 @@ def empty_queues():
 def keep_awake():
     """ If we still have work to do, keep Windows system awake
     """
-    global KERNEL32
-    if KERNEL32 and not sabnzbd.downloader.Downloader.do.paused:
-        if (not PostProcessor.do.empty()) or not NzbQueue.do.is_empty():
-            # set ES_SYSTEM_REQUIRED
-            KERNEL32.SetThreadExecutionState(ctypes.c_int(0x00000001))
+    if KERNEL32 or DARWIN_ML:
+        if not sabnzbd.downloader.Downloader.do.paused:
+            if (not PostProcessor.do.empty()) or not NzbQueue.do.is_empty():
+                if KERNEL32:
+                    # set ES_SYSTEM_REQUIRED
+                    KERNEL32.SetThreadExecutionState(ctypes.c_int(0x00000001))
+                else:
+                    osxmenu.OsxAwake.do.stay_awake = True
 
 
 def CheckFreeSpace():
@@ -788,8 +839,6 @@ def save_data(data, _id, path, do_pickle = True, silent=False):
     except:
         logging.error(Ta('Saving %s failed'), path)
         logging.info("Traceback: ", exc_info = True)
-        return False
-    return misc.has_substance(path)
 
 
 @synchronized(IO_LOCK)
@@ -1009,6 +1058,17 @@ def pid_file(pid_path=None, port=0):
             logging.warning('Cannot access PID file %s', DIR_PID)
 
 
+def check_incomplete_vs_complete():
+    """ Make sure "incomplete" and "complete" are not identical
+    """
+    complete = cfg.complete_dir.get_path()
+    if misc.same_file(cfg.download_dir.get_path(), complete):
+        if misc.real_path('X', cfg.download_dir()) == cfg.download_dir():
+            # Abs path, so set an abs path too
+            cfg.download_dir.set(os.path.join(complete, 'incomplete'))
+        else:
+            cfg.download_dir.set('incomplete')
+
 
 # Required wrapper because nzbstuff.py cannot import downloader.py
 def active_primaries():
