@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -26,7 +26,11 @@ import datetime
 import time
 import cherrypy
 import locale
-locale.setlocale(locale.LC_ALL, "")
+try:
+    locale.setlocale(locale.LC_ALL, "")
+except:
+    # Work-around for Python-ports with bad "locale" support
+    pass
 try:
     import win32api, win32file
 except ImportError:
@@ -47,15 +51,18 @@ from sabnzbd.utils.rsslib import RSS, Item
 from sabnzbd.utils.json import JsonWriter
 from sabnzbd.utils.pathbrowser import folders_at_path
 from sabnzbd.misc import loadavg, to_units, diskfree, disktotal, get_ext, \
-                         get_filename, int_conv, globber, time_format, remove_all
-from sabnzbd.encoding import xml_name, unicoder, special_fixer, platform_encode
+                         get_filename, int_conv, globber, time_format, remove_all, \
+                         starts_with_path
+from sabnzbd.encoding import xml_name, unicoder, special_fixer, platform_encode, html_escape
 from sabnzbd.postproc import PostProcessor
 from sabnzbd.articlecache import ArticleCache
 from sabnzbd.utils.servertests import test_nntp_server_dict
 from sabnzbd.newzbin import Bookmarks
 from sabnzbd.bpsmeter import BPSMeter
 from sabnzbd.database import build_history_info, unpack_history_info, get_history_handle
+import sabnzbd.growler
 import sabnzbd.rss
+import sabnzbd.emailer
 
 
 #------------------------------------------------------------------------------
@@ -228,7 +235,7 @@ def _api_queue_priority(output, value, kwargs):
 def _api_queue_sort(output, value, kwargs):
     """ API: accepts output, sort, dir """
     sort = kwargs.get('sort')
-    direction = kwargs.get('dir')
+    direction = kwargs.get('dir', '')
     if sort:
         sort_queue(sort, direction)
         return report(output)
@@ -239,7 +246,7 @@ def _api_queue_sort(output, value, kwargs):
 def _api_queue_default(output, value, kwargs):
     """ API: accepts output, sort, dir, start, limit """
     sort = kwargs.get('sort')
-    direction = kwargs.get('dir')
+    direction = kwargs.get('dir', '')
     start = kwargs.get('start')
     limit = kwargs.get('limit')
     trans = kwargs.get('trans')
@@ -289,9 +296,14 @@ def _api_addfile(name, output, kwargs):
     else:
         size = len(name.value)
     if name is not None and name.filename and size:
-        sabnzbd.add_nzbfile(name, kwargs.get('pp'), kwargs.get('script'), kwargs.get('cat'),
+        cat = kwargs.get('cat')
+        xcat = kwargs.get('xcat')
+        if not cat and xcat:
+            # Indexer category, so do mapping
+            cat = cat_convert(xcat)
+        res = sabnzbd.add_nzbfile(name, kwargs.get('pp'), kwargs.get('script'), cat,
                             kwargs.get('priority'), kwargs.get('nzbname'))
-        return report(output)
+        return report(output, keyword='', data={'status':res[0]==0, 'nzo_ids' : res[1]}, compat=True)
     else:
         return report(output, _MSG_NO_VALUE)
 
@@ -314,6 +326,8 @@ def _api_retry(name, output, kwargs):
 
 def _api_addlocalfile(name, output, kwargs):
     """ API: accepts name, output, pp, script, cat, priority, nzbname """
+    if name and isinstance(name, list):
+        name = name[0]
     if name:
         if os.path.exists(name):
             fn = get_filename(name)
@@ -321,20 +335,24 @@ def _api_addlocalfile(name, output, kwargs):
                 pp = kwargs.get('pp')
                 script = kwargs.get('script')
                 cat = kwargs.get('cat')
+                xcat = kwargs.get('xcat')
+                if not cat and xcat:
+                    # Indexer category, so do mapping
+                    cat = cat_convert(xcat)
                 priority = kwargs.get('priority')
                 nzbname = kwargs.get('nzbname')
 
                 if get_ext(name) in ('.zip', '.rar'):
-                    sabnzbd.dirscanner.ProcessArchiveFile(\
+                    res = sabnzbd.dirscanner.ProcessArchiveFile(\
                         fn, name, pp=pp, script=script, cat=cat, priority=priority, keep=True)
                 elif get_ext(name) in ('.nzb', '.gz'):
-                    sabnzbd.dirscanner.ProcessSingleFile(\
+                    res = sabnzbd.dirscanner.ProcessSingleFile(\
                         fn, name, pp=pp, script=script, cat=cat, priority=priority, keep=True, nzbname=nzbname)
             else:
                 return report(output, _MSG_NO_FILE)
         else:
             return report(output, _MSG_NO_PATH)
-        return report(output)
+        return report(output, keyword='', data={'status':res[0]==0, 'nzo_ids' : res[1]}, compat=True)
     else:
         return report(output, _MSG_NO_VALUE)
 
@@ -443,41 +461,44 @@ def _api_get_files(name, output, kwargs):
     else:
         return report(output, _MSG_NO_VALUE)
 
-def _api_addurl(name, output, kwargs):
-    """ API: accepts name, output, pp, script, cat, priority, nzbname """
-    if name:
-        sabnzbd.add_url(name, kwargs.get('pp'), kwargs.get('script'), kwargs.get('cat'),
-                        kwargs.get('priority'), kwargs.get('nzbname'))
-        return report(output)
-    else:
-        return report(output, _MSG_NO_VALUE)
-
 
 _RE_NEWZBIN_URL = re.compile(r'/browse/post/(\d+)')
-def _api_addid(name, output, kwargs):
+def _api_addid(names, output, kwargs):
     """ API: accepts name, output, pp, script, cat, priority, nzbname """
     pp = kwargs.get('pp')
     script = kwargs.get('script')
     cat = kwargs.get('cat')
     priority = kwargs.get('priority')
-    nzbname = kwargs.get('nzbname')
+    nzbnames = kwargs.get('nzbname')
+    if not isinstance(names, list):
+        names = [names]
+    if not isinstance(nzbnames, list):
+        nzbnames = [nzbnames]
 
-    newzbin_url = _RE_NEWZBIN_URL.search(name.lower())
+    for n in xrange(len(names)):
+        name = names[n]
+        if n < len(nzbnames):
+            nzbname = nzbnames[n]
+        else:
+            nzbname = ''
 
-    if name:
-        name = name.strip()
-    if name and (name.isdigit() or len(name)==5):
-        sabnzbd.add_msgid(name, pp, script, cat, priority, nzbname)
-        return report(output)
-    elif newzbin_url:
-        sabnzbd.add_msgid(newzbin_url.group(1), pp, script, cat, priority, nzbname)
-        return report(output)
-    elif name:
-        sabnzbd.add_url(name, pp, script, cat, priority, nzbname)
+        newzbin_url = _RE_NEWZBIN_URL.search(name.lower())
+
+        if name:
+            name = name.strip()
+        if name and (name.isdigit() or len(name)==5):
+            sabnzbd.add_msgid(name, pp, script, cat, priority, nzbname)
+        elif newzbin_url:
+            sabnzbd.add_msgid(newzbin_url.group(1), pp, script, cat, priority, nzbname)
+        elif name:
+            sabnzbd.add_url(name, pp, script, cat, priority, nzbname)
+
+    if len(names) > 0:
         return report(output)
     else:
         return report(output, _MSG_NO_VALUE)
 
+
 def _api_pause(name, output, kwargs):
     """ API: accepts output """
     scheduler.plan_resume(0)
@@ -518,7 +539,8 @@ def _api_get_cats(name, output, kwargs):
 
 def _api_get_scripts(name, output, kwargs):
     """ API: accepts output """
-    return report(output, keyword="scripts", data=list_scripts())
+    data = [ unicoder(val) for val in list_scripts() ]
+    return report(output, keyword="scripts", data=data)
 
 
 def _api_version(name, output, kwargs):
@@ -529,15 +551,15 @@ def _api_version(name, output, kwargs):
 def _api_auth(name, output, kwargs):
     """ API: accepts output """
     auth = 'None'
-    if cfg.username() and cfg.password():
-        auth = 'login'
     if not cfg.disable_key():
-        auth = 'apikey'
+        auth = 'badkey'
         key = kwargs.get('key', '')
         if key == cfg.nzb_key():
             auth = 'nzbkey'
         if key == cfg.api_key():
             auth = 'apikey'
+    elif cfg.username() and cfg.password():
+        auth = 'login'
     return report(output, keyword='auth', data=auth)
 
 
@@ -572,8 +594,8 @@ def _api_disconnect(name, output, kwargs):
 
 def _api_osx_icon(name, output, kwargs):
     """ API: accepts output, value """
-    value = kwargs.get('value', '1')
-    sabnzbd.OSX_ICON = int(value != '0')
+    value = kwargs.get('value', '1').strip()
+    cfg.osx_menu.set(value != '0')
     return report(output)
 
 
@@ -586,9 +608,11 @@ def _api_rescan(name, output, kwargs):
 def _api_eval_sort(name, output, kwargs):
     """ API: evaluate sorting expression """
     import sabnzbd.tvsort
+    name = kwargs.get('name', '')
     value = kwargs.get('value', '')
     title = kwargs.get('title')
-    path = sabnzbd.tvsort.eval_sort(name, value, title)
+    multipart = kwargs.get('movieextra', '')
+    path = sabnzbd.tvsort.eval_sort(value, title, name, multipart)
     if path is None:
         return report(output, _MSG_NOT_IMPLEMENTED)
     else:
@@ -607,6 +631,28 @@ def _api_rss_now(name, output, kwargs):
     scheduler.force_rss()
     return report(output)
 
+def _api_reset_quota(name, output, kwargs):
+    """ Reset quota left """
+    BPSMeter.do.reset_quota(force=True)
+
+def _api_test_email(name, output, kwargs):
+    """ API: send a test email, return result """
+    logging.info("Sending testmail")
+    pack = {}
+    pack['download'] = ['action 1', 'action 2']
+    pack['unpack'] = ['action 1', 'action 2']
+    res = sabnzbd.emailer.endjob('I had a d\xe8ja vu', 123, 'unknown', True,
+                                 os.path.normpath(os.path.join(cfg.complete_dir.get_path(), '/unknown/I had a d\xe8ja vu')),
+                                 123*MEBI, pack, 'my_script', 'Line 1\nLine 2\nLine 3\nd\xe8ja vu\n', 0)
+    if res == 'Email succeeded':
+        res = None
+    return report(output, error=res)
+
+def _api_test_notif(name, output, kwargs):
+    """ API: send a test notification, return result """
+    logging.info("Sending test notification")
+    res = sabnzbd.growler.send_notification('SABnzbd', T('Test Notification'), 'other', wait=True)
+    return report(output, error=res)
 
 def _api_undefined(name, output, kwargs):
     """ API: accepts output """
@@ -618,12 +664,14 @@ def _api_browse(name, output, kwargs):
     """ Return tree of local path """
     compact = kwargs.get('compact')
     if compact and compact == '1':
-        limit = int_conv(kwargs.get('limit', 30))
+        paths = []
+        name = platform_encode(kwargs.get('term', ''))
         paths = [entry['path'] for entry in folders_at_path(os.path.dirname(name)) if 'path' in entry]
-        paths = '\n'.join(paths[0:limit])
+        return report(output, keyword='', data=paths)
     else:
+        name = platform_encode(name)
         paths = folders_at_path(name, True)
-    return report(output, keyword='paths', data=paths)
+        return report(output, keyword='paths', data=paths)
 
 
 #------------------------------------------------------------------------------
@@ -721,7 +769,7 @@ _api_table = {
     'fullstatus'      : _api_fullstatus,
     'history'         : _api_history,
     'get_files'       : _api_get_files,
-    'addurl'          : _api_addurl,
+    'addurl'          : _api_addid,
     'addid'           : _api_addid,
     'pause'           : _api_pause,
     'resume'          : _api_resume,
@@ -741,7 +789,10 @@ _api_table = {
     'eval_sort'       : _api_eval_sort,
     'watched_now'     : _api_watched_now,
     'rss_now'         : _api_rss_now,
-    'browse'          : _api_browse
+    'browse'          : _api_browse,
+    'reset_quota'     : _api_reset_quota,
+    'test_email'      : _api_test_email,
+    'test_notif'      : _api_test_notif,
 }
 
 _api_queue_table = {
@@ -769,11 +820,12 @@ _api_config_table = {
 
 
 #------------------------------------------------------------------------------
-def report(output, error=None, keyword='value', data=None, callback=None):
+def report(output, error=None, keyword='value', data=None, callback=None, compat=False):
     """ Report message in json, xml or plain text
         If error is set, only an status/error report is made.
         If no error and no data, only a status report is made.
         Else, a data report is made (optional 'keyword' for outer XML section).
+        'compat' is a special case for compatibility for ascii ouput
     """
     if output == 'json':
         content = "application/json;charset=UTF-8"
@@ -792,6 +844,9 @@ def report(output, error=None, keyword='value', data=None, callback=None):
 
 
     elif output == 'xml':
+        if not keyword:
+            # xml always needs an outer keyword, even when json doesn't
+            keyword = 'result'
         content = "text/xml"
         xmlmaker = xml_factory()
         if error:
@@ -806,13 +861,15 @@ def report(output, error=None, keyword='value', data=None, callback=None):
         content = "text/plain"
         if error:
             response = "error: %s\n" % error
-        elif data is None:
+        elif compat or data is None:
             response = 'ok\n'
         else:
             if type(data) in (list, tuple):
                 # Special handling for list/tuple (backward compatibility)
                 data = [str(val) for val in data]
-                response = '%s\n' % ' '.join(data)
+                data = ' '.join(data)
+            if isinstance(data, unicode):
+                response = u'%s\n' % data
             else:
                 response = '%s\n' % str(data)
 
@@ -937,7 +994,7 @@ def handle_cat_api(output, kwargs):
 
 
 #------------------------------------------------------------------------------
-def build_queue(web_dir=None, root=None, verbose=False, prim=True, verbose_list=None,
+def build_queue(web_dir=None, root=None, verbose=False, prim=True, webdir='', verbose_list=None,
                 dictionary=None, history=False, start=None, limit=None, dummy2=None, trans=False, output=None):
     if output:
         converter = unicoder
@@ -951,7 +1008,7 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, verbose_list=
     else:
         dictn = []
     #build up header full of basic information
-    info, pnfo_list, bytespersec = build_header(prim)
+    info, pnfo_list, bytespersec = build_header(prim, webdir)
     info['isverbose'] = verbose
     cookie = cherrypy.request.cookie
     if cookie.has_key('queue_details'):
@@ -1044,7 +1101,10 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, verbose_list=
         slot['size'] = format_bytes(bytes)
         slot['sizeleft'] = format_bytes(bytesleft)
         if not Downloader.do.paused and status != 'Paused' and status != 'Fetching' and not found_active:
-            slot['status'] = "Downloading"
+            if status == Status.CHECKING:
+                slot['status'] = Status.CHECKING
+            else:
+                slot['status'] = Status.DOWNLOADING
             found_active = True
         else:
             slot['status'] = "%s" % (status)
@@ -1064,7 +1124,7 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, verbose_list=
             slot['percentage'] = "%s" % (int(((mb-mbleft) / mb) * 100))
         slot['missing'] = missing
 
-        if status == 'Paused':
+        if status in (Status.PAUSED, Status.CHECKING):
             slot['timeleft'] = '0:00:00'
             slot['eta'] = 'unknown'
         else:
@@ -1078,7 +1138,11 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, verbose_list=
                 datestart = datetime.datetime.now()
                 slot['eta'] = 'unknown'
 
-        slot['avg_age'] = calc_age(average_date, bool(trans))
+        if status == Status.GRABBING:
+            slot['avg_age'] = '---'
+        else:
+            slot['avg_age'] = calc_age(average_date, bool(trans))
+
         slot['verbosity'] = ""
         if web_dir:
             finished = []
@@ -1165,6 +1229,16 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, verbose_list=
     return info, pnfo_list, bytespersec, verbose_list, dictn
 
 
+#------------------------------------------------------------------------------
+def fast_queue():
+    """ Return paused, bytes_left, bpsnow, time_left """
+    bytes_left = NzbQueue.do.remaining()
+    paused = Downloader.do.paused
+    bpsnow = BPSMeter.do.get_bps()
+    time_left = calc_timeleft(bytes_left, bpsnow)
+    return paused, bytes_left, bpsnow, time_left
+
+
 #------------------------------------------------------------------------------
 def qstatus_data():
     """Build up the queue status as a nested object and output as a JSON object
@@ -1192,16 +1266,16 @@ def qstatus_data():
 
     state = "IDLE"
     if Downloader.do.paused:
-        state = "PAUSED"
+        state = Status.PAUSED
     elif qnfo[QNFO_BYTES_LEFT_FIELD] / MEBI > 0:
-        state = "DOWNLOADING"
+        state = Status.DOWNLOADING
 
     status = {
         "state" : state,
         "paused" : Downloader.do.paused,
         "pause_int" : scheduler.pause_int(),
-        "kbpersec" : BPSMeter.do.get_bps() / KIBI,
-        "speed" : to_units(BPSMeter.do.get_bps(), dec_limit=1),
+        "kbpersec" : bpsnow / KIBI,
+        "speed" : to_units(bpsnow, dec_limit=1),
         "mbleft" : qnfo[QNFO_BYTES_LEFT_FIELD] / MEBI,
         "mb" : qnfo[QNFO_BYTES_FIELD] / MEBI,
         "noofslots" : len(pnfo_list),
@@ -1430,23 +1504,26 @@ _SKIN_CACHE = {}    # Stores pre-translated acronyms
 def Ttemplate(txt):
     """ Translation function for Skin texts
     """
-    return _SKIN_CACHE.get(txt, txt)
+    global _SKIN_CACHE
+    if txt in _SKIN_CACHE:
+        return _SKIN_CACHE[txt]
+    else:
+        tra = html_escape(Tx(SKIN_TEXT.get(txt, txt)))
+        _SKIN_CACHE[txt] = tra
+        return tra
 
 
-def cache_skin_trans():
-    """ Build cache for skin translations
+def clear_trans_cache():
+    """ Clean cache for skin translations
     """
     global _SKIN_CACHE
-    for txt in SKIN_TEXT:
-        _SKIN_CACHE[txt] = Tx(SKIN_TEXT.get(txt, txt))
+    dummy = _SKIN_CACHE
+    _SKIN_CACHE = {}
+    del dummy
+    sabnzbd.WEBUI_READY = True
 
-def check_trans():
-    """ Check whether language has been initialized
-    """
-    global _SKIN_CACHE
-    return bool(_SKIN_CACHE)
 
-def build_header(prim):
+def build_header(prim, webdir=''):
     try:
         uptime = calc_age(sabnzbd.START)
     except:
@@ -1478,10 +1555,12 @@ def build_header(prim):
     header['last_warning'] = sabnzbd.GUIHANDLER.last().replace('WARNING', Ta('WARNING:')).replace('ERROR', Ta('ERROR:'))
     header['active_lang'] = cfg.language()
     header['newzbin_url'] = cfg.newzbin_url()
-    if prim:
-        header['webdir'] = sabnzbd.WEB_DIR
-    else:
-        header['webdir'] = sabnzbd.WEB_DIR2
+    header['my_lcldata'] = sabnzbd.DIR_LCLDATA
+    header['my_home'] = sabnzbd.DIR_HOME
+
+
+    header['webdir'] = webdir
+    header['newzbin_url'] = cfg.newzbin_url()
 
     header['finishaction'] = sabnzbd.QUEUECOMPLETE
     header['nt'] = sabnzbd.WIN32
@@ -1489,6 +1568,7 @@ def build_header(prim):
     header['power_options'] = sabnzbd.WIN32 or sabnzbd.DARWIN or sabnzbd.LINUX_POWER
 
     header['session'] = cfg.api_key()
+    header['uniconfig'] = cfg.uniconfig() and sabnzbd.WEB_DIRC
 
     bytespersec = BPSMeter.do.get_bps()
     qnfo = NzbQueue.do.queue_info()
@@ -1502,12 +1582,15 @@ def build_header(prim):
     header['mb']       = "%.2f" % (bytes / MEBI)
     header['sizeleft']   = format_bytes(bytesleft)
     header['size']       = format_bytes(bytes)
+    header['quota'] = to_units(BPSMeter.do.quota)
+    header['have_quota'] = bool(BPSMeter.do.quota > 0.0)
+    header['left_quota'] = to_units(BPSMeter.do.left)
 
     status = ''
     if Downloader.do.paused:
-        status = 'Paused'
+        status = Status.PAUSED
     elif bytespersec > 0:
-        status = 'Downloading'
+        status = Status.DOWNLOADING
     else:
         status = 'Idle'
     header['status'] = "%s" % status
@@ -1648,7 +1731,7 @@ def build_history(start=None, limit=None, verbose=False, verbose_list=None, sear
         item['retry'] = int(bool(item.get('status') == 'Failed' and \
                                  path and \
                                  path not in retry_folders and \
-                                 path.startswith(cfg.download_dir.get_path()) and \
+                                 starts_with_path(path, cfg.download_dir.get_path()) and \
                                  os.path.exists(path)) and \
                                  not bool(globber(os.path.join(path, JOB_ADMIN), 'SABnzbd_n*')) \
                                  )
@@ -1788,6 +1871,7 @@ def list_scripts(default=False):
             if os.path.isfile(script):
                 if (sabnzbd.WIN32 and os.path.splitext(script)[1].lower() in PATHEXT and \
                                       not (win32api.GetFileAttributes(script) & win32file.FILE_ATTRIBUTE_HIDDEN)) or \
+                   script.endswith('.py') or \
                    (not sabnzbd.WIN32 and os.access(script, os.X_OK) and not os.path.basename(script).startswith('.')):
                     lst.append(os.path.basename(script))
         lst.insert(0, 'None')
