@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -34,13 +34,15 @@ except:
     new_md5 = md5.new
 
 import sabnzbd
-from sabnzbd.misc import get_filepath, sanitize_filename, get_unique_path, renamer
+from sabnzbd.misc import get_filepath, sanitize_filename, get_unique_path, renamer, \
+                         set_permissions, flag_file
+from sabnzbd.constants import QCHECK_FILE
 import sabnzbd.cfg as cfg
 from sabnzbd.articlecache import ArticleCache
 from sabnzbd.postproc import PostProcessor
 import sabnzbd.downloader
 from sabnzbd.utils.rarfile import RarFile, is_rarfile
-from sabnzbd.encoding import latin1, unicoder
+from sabnzbd.encoding import latin1, unicoder, is_utf8
 
 
 #------------------------------------------------------------------------------
@@ -59,18 +61,18 @@ class Assembler(Thread):
     def stop(self):
         self.process(None)
 
-    def process(self, nzf):
-        self.queue.put(nzf)
+    def process(self, job):
+        self.queue.put(job)
 
     def run(self):
         import sabnzbd.nzbqueue
         while 1:
-            nzo_nzf_tuple = self.queue.get()
-            if not nzo_nzf_tuple:
+            job = self.queue.get()
+            if not job:
                 logging.info("Shutting down")
                 break
 
-            nzo, nzf = nzo_nzf_tuple
+            nzo, nzf = job
 
             if nzf:
                 sabnzbd.CheckFreeSpace()
@@ -100,7 +102,7 @@ class Assembler(Thread):
                     nzf.remove_admin()
                     setname = nzf.setname
                     if nzf.is_par2 and (nzo.md5packs.get(setname) is None):
-                        pack = GetMD5Hashes(filepath)
+                        pack = GetMD5Hashes(filepath)[0]
                         if pack:
                             nzo.md5packs[setname] = pack
                             logging.debug('Got md5pack for set %s', setname)
@@ -173,6 +175,7 @@ def _assemble(nzf, path, dupe):
 
     fout.flush()
     fout.close()
+    set_permissions(path)
     if md5:
         nzf.md5sum = md5.digest()
         del md5
@@ -200,32 +203,35 @@ def file_has_articles(nzf):
 
 def GetMD5Hashes(fname):
     """ Get the hash table from a PAR2 file
-        Return as dictionary, indexed on names
+        Return as dictionary, indexed on names and True for utf8-encoded names
     """
+    new_encoding = False
     table = {}
-    try:
-        f = open(fname, 'rb')
-    except:
-        return table
+    if not flag_file(os.path.split(fname)[0], QCHECK_FILE):
+        try:
+            f = open(fname, 'rb')
+        except:
+            return table, new_encoding
 
-    try:
-        header = f.read(8)
-        while header:
-            name, hash = ParseFilePacket(f, header)
-            if name:
-                table[name] = hash
+        try:
             header = f.read(8)
+            while header:
+                name, hash = ParseFilePacket(f, header)
+                new_encoding |= is_utf8(name)
+                if name:
+                    table[name] = hash
+                header = f.read(8)
 
-    except (struct.error, IndexError):
-        logging.info('Cannot use corrupt par2 file for QuickCheck, "%s"', fname)
-        table = {}
-    except:
-        logging.debug('QuickCheck parser crashed in file %s', fname)
-        logging.info('Traceback: ', exc_info = True)
-        table = {}
+        except (struct.error, IndexError):
+            logging.info('Cannot use corrupt par2 file for QuickCheck, "%s"', fname)
+            table = {}
+        except:
+            logging.debug('QuickCheck parser crashed in file %s', fname)
+            logging.info('Traceback: ', exc_info = True)
+            table = {}
 
-    f.close()
-    return table
+        f.close()
+    return table, new_encoding
 
 
 def ParseFilePacket(f, header):
