@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -65,7 +65,7 @@ else:
 # Configuration instances
 #
 quick_check = OptionBool('misc', 'quick_check', True)
-fail_on_crc = OptionBool('misc', 'fail_on_crc', False)
+fail_on_crc = OptionBool('misc', 'fail_on_crc', True)
 send_group = OptionBool('misc', 'send_group', False)
 sfv_check = OptionBool('misc', 'sfv_check', True)
 
@@ -79,10 +79,11 @@ email_full    = OptionBool('misc', 'email_full', False)
 email_dir     = OptionDir('misc', 'email_dir', create=True)
 email_rss     = OptionBool('misc', 'email_rss', False)
 
-version_check = OptionBool('misc', 'check_new_rel', True)
+version_check = OptionNumber('misc', 'check_new_rel', 1)
 autobrowser = OptionBool('misc', 'auto_browser', True)
 replace_illegal = OptionBool('misc', 'replace_illegal', True)
 pre_script = OptionStr('misc', 'pre_script', 'None')
+start_paused = OptionBool('misc', 'start_paused', False)
 
 enable_unrar = OptionBool('misc', 'enable_unrar', True)
 enable_unzip = OptionBool('misc', 'enable_unzip', True)
@@ -99,6 +100,8 @@ ignore_wrong_unrar = OptionBool('misc', 'ignore_wrong_unrar', False)
 par2_multicore = OptionBool('misc', 'par2_multicore', True)
 allow_64bit_tools = OptionBool('misc', 'allow_64bit_tools', True)
 allow_streaming = OptionBool('misc', 'allow_streaming', False)
+pre_check = OptionBool('misc', 'pre_check', False)
+req_completion_rate = OptionNumber('misc', 'req_completion_rate', 100.2, 100, 200)
 
 newzbin_username = OptionStr('newzbin', 'username')
 newzbin_password = OptionPassword('newzbin', 'password')
@@ -133,6 +136,7 @@ enable_tv_sorting = OptionBool('misc', 'enable_tv_sorting', False)
 tv_sort_string = OptionStr('misc', 'tv_sort_string')
 tv_sort_countries = OptionNumber('misc', 'tv_sort_countries', 1)
 tv_categories = OptionList('misc', 'tv_categories', '')
+movie_rename_limit = OptionStr('misc', 'movie_rename_limit', '100M')
 
 enable_movie_sorting = OptionBool('misc', 'enable_movie_sorting', False)
 movie_sort_string = OptionStr('misc', 'movie_sort_string')
@@ -147,6 +151,8 @@ date_categories = OptionStr('misc', 'date_categories', ['tv'])
 matrix_username = OptionStr('nzbmatrix', 'username')
 matrix_apikey = OptionStr('nzbmatrix', 'apikey')
 matrix_del_bookmark = OptionBool('nzbmatrix', 'del_bookmark', True)
+xxx_username = OptionStr('nzbxxx', 'username')
+xxx_apikey = OptionStr('nzbxxx', 'apikey')
 
 configlock = OptionBool('misc', 'config_lock', 0)
 
@@ -162,8 +168,9 @@ admin_dir = OptionDir('misc', 'admin_dir', DEF_ADMIN_DIR, validation=validate_sa
 #log_dir = OptionDir('misc', 'log_dir', 'logs')
 dirscan_dir = OptionDir('misc', 'dirscan_dir', create=False)
 dirscan_speed = OptionNumber('misc', 'dirscan_speed', DEF_SCANRATE, 0, 3600)
-SIZE_LIMIT = OptionStr('misc', 'size_limit')
+size_limit = OptionStr('misc', 'size_limit', '0')
 password_file = OptionDir('misc', 'password_file', '', create=False)
+fsys_type = OptionNumber('misc', 'fsys_type', 0, 0, 2)
 
 cherryhost = OptionStr('misc', 'host', DEF_HOST)
 if sabnzbd.WIN32:
@@ -191,7 +198,7 @@ warned_old_queue = OptionBool('misc', 'warned_old_queue', False)
 
 log_web = OptionBool('logging', 'enable_cherrypy_logging', False)
 log_dir = OptionDir('misc', 'log_dir', 'logs', validation=validate_notempty)
-log_level = OptionNumber('logging', 'log_level', 1, 0, 2)
+log_level = OptionNumber('logging', 'log_level', 1, -1, 2)
 log_size = OptionStr('logging', 'max_log_size', '5242880')
 log_backups = OptionNumber('logging', 'log_backups', 5, 1, 1024)
 log_new = OptionBool('logging', 'log_new', False)
@@ -204,8 +211,7 @@ language = OptionStr('misc', 'language', 'en')
 ssl_type = OptionStr('misc', 'ssl_type', 'v23')
 unpack_check = OptionBool('misc', 'unpack_check', True)
 no_penalties = OptionBool('misc', 'no_penalties', False)
-
-growl_enable = OptionBool('growl', 'growl_enable', True)
+randomize_server_ip = OptionBool('misc', 'randomize_server_ip', False)
 
 # Internal options, not saved in INI file
 debug_delay = OptionNumber('misc', 'debug_delay', 0, add=False)
@@ -214,7 +220,28 @@ api_key = OptionStr('misc', 'api_key', create_api_key())
 nzb_key = OptionStr('misc', 'nzb_key', create_api_key())
 disable_key = OptionBool('misc', 'disable_api_key', False)
 api_warnings = OptionBool('misc', 'api_warnings', True)
+max_art_tries = OptionNumber('misc', 'max_art_tries', 3, 2)
+max_art_opt = OptionBool('misc', 'max_art_opt', False)
 use_pickle = OptionBool('misc', 'use_pickle', False)
+no_ipv6 = OptionBool('misc', 'no_ipv6', False)
+
+growl_server = OptionStr('growl', 'growl_server')
+growl_password = OptionPassword('growl', 'growl_password')
+growl_enable = OptionBool('growl', 'growl_enable', True)
+ntfosd_enable = OptionBool('growl', 'ntfosd_enable', not sabnzbd.WIN32 and not sabnzbd.DARWIN)
+
+quota_size = OptionStr('misc', 'quota_size')
+quota_day = OptionStr('misc', 'quota_day')
+quota_resume = OptionBool('misc', 'quota_resume', False)
+quota_period = OptionStr('misc', 'quota_period', 'm')
+
+osx_menu = OptionBool('misc', 'osx_menu', True)
+osx_speed = OptionBool('misc', 'osx_speed', True)
+win_menu = OptionBool('misc', 'win_menu', True)
+uniconfig = OptionBool('misc', 'uniconfig', True)
+allow_incomplete_nzb = OptionBool('misc', 'allow_incomplete_nzb', False)
+marker_file = OptionStr('misc', 'nomedia_marker', '')
+wait_ext_drive = OptionNumber('misc', 'wait_ext_drive', 5, 1, 60)
 
 #------------------------------------------------------------------------------
 # Set root folders for Folder config-items
