@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -62,6 +62,8 @@ SCAN_FILE_NAME    = 'watched_data.sab'
 TERM_FLAG_FILE    = 'running.sab'
 FUTURE_Q_FOLDER   = 'future'
 JOB_ADMIN = '__ADMIN__'
+VERIFIED_FILE = '__verified__'
+QCHECK_FILE = '__skip_qcheck__'
 ATTRIB_FILE = 'SABnzbd_attrib'
 REPAIR_REQUEST = 'repair-all.sab'
 
@@ -81,6 +83,7 @@ DEF_LANGUAGE     = 'locale'
 DEF_INTERFACES   = 'interfaces'
 DEF_INT_LANGUAGE = 'locale'
 DEF_EMAIL_TMPL   = 'email'
+DEF_STDCONFIG    = 'Config'
 DEF_STDINTF      = 'Plush'
 DEF_SKIN_COLORS  = {'smpl':'white', 'classic':'darkblue', 'mobile':'light', 'plush' : 'gold'}
 DEF_MAIN_TMPL    = 'templates/main.tmpl'
@@ -112,6 +115,7 @@ LOW_PRIORITY = -1
 DEFAULT_PRIORITY = -100
 PAUSED_PRIORITY = -2
 DUP_PRIORITY = -3
+STOP_PRIORITY = -4
 
 #(MATCHER, [EXTRA,MATCHERS])
 series_match = [ (r'( [sS]|[\d]+)x(\d+)', # 1x01
@@ -136,4 +140,20 @@ date_match = [r'(\d{4})\W(\d{1,2})\W(\d{1,2})', #2008-10-16
 
 year_match = r'[\W]([1|2]\d{3})([^\w]|$)' # Something '(YYYY)' or '.YYYY.' or ' YYYY '
 
-sample_match = r'((^|[\W_])sample\d*[\W_])|(-s\.)' # something-sample.avi something-s.avi
+sample_match = r'((^|[\W_])sample\d*[\W_])|(-s\.\w+$)' # something-sample.avi something-s.avi
+
+class Status():
+    COMPLETED = 'Completed'
+    CHECKING = 'Checking'
+    DOWNLOADING = 'Downloading'
+    EXTRACTING = 'Extracting'
+    FAILED = 'Failed'
+    FETCHING = 'Fetching'
+    GRABBING = 'Grabbing'
+    MOVING = 'Moving'
+    PAUSED = 'Paused'
+    QUEUED = 'Queued'
+    QUICK_CHECK = 'QuickCheck'
+    REPAIRING = 'Repairing'
+    RUNNING = 'Running'
+    VERIFYING = 'Verifying'
