@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -37,6 +37,7 @@ from sabnzbd.articlecache import ArticleCache
 import sabnzbd.downloader
 import sabnzbd.cfg as cfg
 from sabnzbd.encoding import name_fixer
+from sabnzbd.misc import match_str
 
 #-------------------------------------------------------------------------------
 
@@ -84,15 +85,20 @@ class Decoder(Thread):
 
             data = None
 
-            register = True
+            register = True  # Finish article
+            found = False    # Proper article found
 
             if lines:
                 logme = None
                 try:
+                    if nzo.precheck:
+                        raise BadYenc
+                    register = True
                     logging.debug("Decoding %s", article)
 
                     data = decode(article, lines)
                     nzf.article_count += 1
+                    found = True
                 except IOError, e:
                     logme = Ta('Decoding %s failed') % article
                     logging.info(logme)
@@ -116,11 +122,31 @@ class Decoder(Thread):
                             register = False
                             logme = None
 
-                except BadYenc, e:
-                    logme = Ta('Badly formed yEnc article in %s') % article
-                    logging.info(logme)
+                except BadYenc:
+                    # Handles precheck and badly formed articles
+                    killed = False
+                    found = False
+                    if nzo.precheck and lines and lines[0].startswith('223 '):
+                        # STAT was used, so we only get a status code
+                        found = True
+                    else:
+                        # Examine headers (for precheck) or body (for download)
+                        # And look for DMCA clues (while skipping "X-" headers)
+                        for line in lines:
+                            if not line.startswith('X-') and match_str(line.lower(), ('dmca', 'removed', 'cancel', 'blocked')):
+                                logging.info('Article removed from server (%s)', article)
+                                killed = True
+                                break
+                    if nzo.precheck:
+                        if found or not killed:
+                            # Pre-check, proper article found, just register
+                            logging.debug('Server has article %s', article)
+                            register = True
+                    elif not killed and not found:
+                        logme = Ta('Badly formed yEnc article in %s') % article
+                        logging.info(logme)
 
-                    if cfg.fail_on_crc():
+                    if not found:
                         new_server_found = self.__search_new_server(article)
                         if new_server_found:
                             register = False
@@ -129,6 +155,12 @@ class Decoder(Thread):
                 except:
                     logme = Ta('Unknown Error while decoding %s') % article
                     logging.info(logme)
+                    logging.info("Traceback: ", exc_info = True)
+
+                    new_server_found = self.__search_new_server(article)
+                    if new_server_found:
+                        register = False
+                        logme = None
 
                 if logme:
                     article.nzf.nzo.inc_log('bad_art_log', logme)
@@ -137,12 +169,14 @@ class Decoder(Thread):
                 new_server_found = self.__search_new_server(article)
                 if new_server_found:
                     register = False
+                elif nzo.precheck:
+                    found = False
 
             if data:
                 ArticleCache.do.save_article(article, data)
 
             if register:
-                NzbQueue.do.register_article(article)
+                NzbQueue.do.register_article(article, found)
 
     def __search_new_server(self, article):
         from sabnzbd.nzbqueue import NzbQueue
@@ -169,6 +203,7 @@ class Decoder(Thread):
 
         if new_server_found:
             article.fetcher = None
+            article.tries = 0
 
             ## Allow all servers to iterate over this nzo and nzf again ##
             NzbQueue.do.reset_try_lists(nzf, nzo)
@@ -198,19 +233,22 @@ def decode(article, data):
         #Deal with non-yencoded posts
         if not ybegin:
             found = False
-            for i in xrange(10):
-                if data[i].startswith('begin '):
-                    nzf.filename = name_fixer(data[i].split(None, 2)[2])
-                    nzf.type = 'uu'
-                    found = True
-                    break
-            if found:
-                for n in xrange(i):
-                    data.pop(0)
-            if data[-1] == 'end':
-                data.pop()
-                if data[-1] == '`':
+            try:
+                for i in xrange(10):
+                    if data[i].startswith('begin '):
+                        nzf.filename = name_fixer(data[i].split(None, 2)[2])
+                        nzf.type = 'uu'
+                        found = True
+                        break
+                if found:
+                    for n in xrange(i+1):
+                        data.pop(0)
+                if data[-1] == 'end':
                     data.pop()
+                    if data[-1] == '`':
+                        data.pop()
+            except IndexError:
+                raise BadYenc()
 
             decoded_data = '\r\n'.join(data)
 
@@ -260,7 +298,7 @@ def yCheck(data):
     yend = None
 
     ## Check head
-    for i in xrange(10):
+    for i in xrange(40):
         try:
             if data[i].startswith('=ybegin '):
                 splits = 3
