@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -62,9 +62,11 @@ def CompareStat(tup1, tup2):
 def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False, priority=None, url=''):
     """ Analyse ZIP file and create job(s).
         Accepts ZIP files with ONLY nzb/nfo/folder files in it.
-        returns: -1==Error/Retry, 0==OK, 1==Ignore
+        returns (status, nzo_ids)
+            status: -1==Error/Retry, 0==OK, 1==Ignore
     """
     from sabnzbd.nzbqueue import add_nzo
+    nzo_ids = []
     if catdir is None:
         catdir = cat
 
@@ -74,14 +76,14 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
         try:
             zf = zipfile.ZipFile(path)
         except:
-            return -1
+            return -1, []
     elif is_rarfile(path):
         try:
             zf = RarFile(path)
         except:
-            return -1
+            return -1, []
     else:
-        return 1
+        return 1, []
 
     status = 1
     names = zf.namelist()
@@ -100,18 +102,17 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
                     data = zf.read(name)
                 except:
                     zf.close()
-                    return -1
+                    return -1, []
                 name = re.sub(r'\[.*nzbmatrix.com\]', '', name)
                 name = os.path.basename(name)
                 name = misc.sanitize_foldername(name)
                 if data:
                     try:
-                        nzo = nzbstuff.NzbObject(name, 0, pp, script, data, cat=cat, priority=priority)
-                        nzo.url = url
+                        nzo = nzbstuff.NzbObject(name, 0, pp, script, data, cat=cat, url=url, priority=priority)
                     except:
                         nzo = None
                     if nzo:
-                        add_nzo(nzo)
+                        nzo_ids.append(add_nzo(nzo))
         zf.close()
         try:
             if not keep: os.remove(path)
@@ -123,16 +124,18 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
         zf.close()
         status = 1
 
-    return status
+    return status, nzo_ids
 
 
 def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False,
                       priority=None, nzbname=None, reuse=False, nzo_info=None, dup_check=True, url=''):
     """ Analyse file and create a job from it
         Supports NZB, NZB.GZ and GZ.NZB-in-disguise
-        returns: -2==Error/retry, -1==Error, 0==OK, 1==OK-but-ignorecannot-delete
+        returns (status, nzo_ids)
+            status: -2==Error/retry, -1==Error, 0==OK, 1==OK-but-ignorecannot-delete
     """
     from sabnzbd.nzbqueue import add_nzo
+    nzo_ids = []
     if catdir is None:
         catdir = cat
 
@@ -154,7 +157,7 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
     except:
         logging.warning(Ta('Cannot read %s'), path)
         logging.info("Traceback: ", exc_info = True)
-        return -2
+        return -2, nzo_ids
 
 
     if name:
@@ -164,28 +167,27 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
 
     try:
         nzo = nzbstuff.NzbObject(name, 0, pp, script, data, cat=cat, priority=priority, nzbname=nzbname,
-                                 nzo_info=nzo_info, reuse=reuse, dup_check=dup_check)
-        nzo.url = url
+                                 nzo_info=nzo_info, url=url, reuse=reuse, dup_check=dup_check)
     except TypeError:
         # Duplicate, ignore
         nzo = None
     except:
         if data.find("<nzb") >= 0 and data.find("</nzb") < 0:
             # Looks like an incomplete file, retry
-            return -2
+            return -2, nzo_ids
         else:
-            return -1
+            return -1, nzo_ids
 
     if nzo:
-        add_nzo(nzo)
+        nzo_ids.append(add_nzo(nzo))
     try:
         if not keep: os.remove(path)
     except:
         logging.error(Ta('Error removing %s'), path)
         logging.info("Traceback: ", exc_info = True)
-        return 1
+        return 1, nzo_ids
 
-    return 0
+    return 0, nzo_ids
 
 
 def CleanList(list, folder, files):
@@ -287,7 +289,7 @@ class DirScanner(threading.Thread):
 
             for filename in files:
                 path = os.path.join(folder, filename)
-                if os.path.isdir(path) or path in self.ignored:
+                if os.path.isdir(path) or path in self.ignored or filename[0] == '.':
                     continue
 
                 ext = os.path.splitext(path)[1].lower()
@@ -330,7 +332,7 @@ class DirScanner(threading.Thread):
 
                     # Handle ZIP files, but only when containing just NZB files
                     if ext in ('.zip', '.rar') :
-                        res = ProcessArchiveFile(filename, path, catdir=catdir)
+                        res, nzo_ids = ProcessArchiveFile(filename, path, catdir=catdir, url=path)
                         if res == -1:
                             self.suspected[path] = stat_tuple
                         elif res == 0:
@@ -340,7 +342,7 @@ class DirScanner(threading.Thread):
 
                     # Handle .nzb, .nzb.gz or gzip-disguised-as-nzb
                     elif ext == '.nzb' or filename.lower().endswith('.nzb.gz'):
-                        res = ProcessSingleFile(filename, path, catdir=catdir)
+                        res, nzo_id = ProcessSingleFile(filename, path, catdir=catdir, url=path)
                         if res < 0:
                             self.suspected[path] = stat_tuple
                         elif res == 0:
