@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -191,9 +191,16 @@ def endjob(filename, msgid, cat, status, path, bytes, stages, script, script_out
     """ Send end-of-job email """
 
     # Translate the stage names
+    tr = sabnzbd.api.Ttemplate
     xstages = {}
     for stage in stages:
-        xstages[Tx('stage-'+stage.lower())] = stages[stage]
+        lines = []
+        for line in stages[stage]:
+            if '\n' in line or '<br/>' in line:
+                lines.extend(line.replace('<br/>', '\n').split('\n'))
+            else:
+                lines.append(line)
+        xstages[tr('stage-'+stage.lower())] = lines
 
     parm = {}
     parm['status'] = status
@@ -218,6 +225,13 @@ def rss_mail(feed, jobs):
     return send_with_template('rss', parm)
 
 
+def badfetch_mail(msg, url):
+    """ Send notification email about failed NZB fetch """
+
+    parm = {'url' : url, 'msg' : msg}
+    return send_with_template('badfetch', parm)
+
+
 ################################################################################
 # EMAIL_DISKFULL
 #
