@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -31,13 +31,19 @@ import socket
 import time
 import glob
 import stat
+try:
+    socket.ssl
+    _HAVE_SSL = True
+except:
+    _HAVE_SSL = False
 
 import sabnzbd
 from sabnzbd.decorators import synchronized
-from sabnzbd.constants import DEFAULT_PRIORITY, FUTURE_Q_FOLDER, JOB_ADMIN, GIGI
+from sabnzbd.constants import DEFAULT_PRIORITY, FUTURE_Q_FOLDER, JOB_ADMIN, GIGI, VERIFIED_FILE, Status
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 from sabnzbd.encoding import unicoder, latin1
+import sabnzbd.growler as growler
 
 RE_VERSION = re.compile('(\d+)\.(\d+)\.(\d+)([a-zA-Z]*)(\d*)')
 RE_UNITS = re.compile('(\d+\.*\d*)\s*([KMGTP]{0,1})', re.I)
@@ -252,6 +258,23 @@ def sanitize_foldername(name):
     return name
 
 
+#------------------------------------------------------------------------------
+def flag_file(path, flag, create=False):
+    """ Create verify flag file or return True if it already exists """
+    path = os.path.join(path, JOB_ADMIN)
+    path = os.path.join(path, flag)
+    if create:
+        try:
+            f = open(path, 'w')
+            f.write('ok\n')
+            f.close()
+            return True
+        except IOError:
+            return False
+    else:
+        return os.path.exists(path)
+
+
 ################################################################################
 # DirPermissions                                                               #
 ################################################################################
@@ -294,6 +317,8 @@ def real_path(loc, path):
         When 'path' is absolute, return normalized path
         A path starting with ~ will be located in the user's Home folder
     """
+    # The Windows part is a bit convoluted because
+    # os.path.join() doesn't behave the same for all Python versions
     if path:
         path = path.strip()
     else:
@@ -302,9 +327,15 @@ def real_path(loc, path):
         if not sabnzbd.WIN32 and path.startswith('~/'):
             path = path.replace('~', sabnzbd.DIR_HOME, 1)
         if sabnzbd.WIN32:
-            if path[0].isalpha() and len(path) > 1 and path[1] == ':':
-                if len(path) == 2 or path[2] not in '\\/':
+            path = path.replace('/', '\\')
+            if len(path) > 1 and path[0].isalpha() and path[1] == ':':
+                if len(path) == 2 or path[2] != '\\':
                     path = path.replace(':', ':\\', 1)
+            elif path.startswith('\\\\'):
+                pass
+            elif path.startswith('\\'):
+                if len(loc) > 1 and loc[0].isalpha() and loc[1] == ':':
+                    path = loc[:2] + path
             else:
                 path = os.path.join(loc, path)
         elif path[0] != '/':
@@ -559,11 +590,20 @@ def check_latest_version():
 
     logging.debug("Checked for a new release, cur= %s, latest= %s (on %s)", current, latest, url)
 
+    if latest_test and cfg.version_check() > 1:
+        # User always wants to see the latest test release
+        latest = latest_test
+        url = url_beta
+
     if testver and current < latest:
+        # This is a test version, but user has't seen the
+        # "Final" of this one yet, so show the Final
         sabnzbd.NEW_VERSION = "%s;%s" % (latest_label, url)
     elif current < latest:
+        # This one is behind, show latest final
         sabnzbd.NEW_VERSION = "%s;%s" % (latest_label, url)
     elif testver and current < latest_test:
+        # This is a test version beyond the latest Final, so show latest Alpha/Beta/RC
         sabnzbd.NEW_VERSION = "%s;%s" % (latest_testlabel, url_beta)
 
 
@@ -598,9 +638,12 @@ def to_units(val, spaces=0, dec_limit=2):
         dig_limit==2 show single decimal for G and higher
     """
     decimals = 0
-    val = str(val).strip()
-    if val == "-1":
-        return val
+    if val < 0:
+        sign = '-'
+    else:
+        sign = ''
+    val = str(abs(val)).strip()
+
     n = 0
     try:
         val = float(val)
@@ -617,25 +660,30 @@ def to_units(val, spaces=0, dec_limit=2):
     else:
         decimals = 0
 
-    format = '%%.%sf %%s' % decimals
-    return format % (val, unit)
+    format = '%%s%%.%sf %%s' % decimals
+    return format % (sign, val, unit)
 
 #------------------------------------------------------------------------------
 def same_file(a, b):
-    """ Return True if both paths are identical """
+    """ Return 0 if A and B have nothing in common
+        return 1 if A and B are actually the same path
+        return 2 if B is a subfolder of A
+    """
+    a = os.path.normpath(os.path.abspath(a))
+    b = os.path.normpath(os.path.abspath(b))
+    if sabnzbd.WIN32 or sabnzbd.DARWIN:
+        a = a.lower()
+        b = b.lower()
 
+    if b.startswith(a):
+        return 2
     if "samefile" in os.path.__dict__:
         try:
-            return os.path.samefile(a, b)
+            return int(os.path.samefile(a, b))
         except:
-            return False
+            return 0
     else:
-        try:
-            a = os.path.normpath(os.path.abspath(a)).lower()
-            b = os.path.normpath(os.path.abspath(b)).lower()
-            return a == b
-        except:
-            return False
+        return int(a == b)
 
 #------------------------------------------------------------------------------
 def exit_sab(value):
@@ -643,6 +691,10 @@ def exit_sab(value):
     """
     sys.stderr.flush()
     sys.stdout.flush()
+    if getattr(sys, 'frozen', None) == 'macosx_app':
+        sabnzbd.SABSTOP = True
+        from PyObjCTools import AppHelper
+        AppHelper.stopEventLoop()
     sys.exit(value)
 
 
@@ -664,18 +716,36 @@ def split_host(srv):
     return (host, port)
 
 
+#------------------------------------------------------------------------------
+def hostname():
+    """ Return host's pretty name """
+    if sabnzbd.WIN32:
+        return os.environ.get('computername', 'unknown')
+    try:
+        return os.uname()[1]
+    except:
+        return 'unknown'
+
+
 #------------------------------------------------------------------------------
 def check_mount(path):
     """ Return False if volume isn't mounted on Linux or OSX
+        Retry 6 times with an interval of 1 sec.
     """
     if sabnzbd.DARWIN:
         m = re.search(r'^(/Volumes/[^/]+)/', path, re.I)
-    elif not sabnzbd.WIN32:
-        m = re.search(r'^(/(?:mnt|media)/[^/]+)/', path)
+    elif sabnzbd.WIN32:
+        m = re.search(r'^([a-z]:\\)', path, re.I)
     else:
-        m = None
-    return (not m) or os.path.exists(m.group(1))
+        m = re.search(r'^(/(?:mnt|media)/[^/]+)/', path)
 
+    if m:
+        for n in xrange(cfg.wait_ext_drive() or 1):
+            if os.path.exists(m.group(1)):
+                return True
+            logging.debug('Waiting for %s to come online', m.group(1))
+            time.sleep(1)
+    return not m
 
 #------------------------------------------------------------------------------
 # Locked directory operations
@@ -745,8 +815,9 @@ def move_to_path(path, new_path, unique=True):
                 shutil.copyfile(path, new_path)
                 os.remove(path)
             except:
-                logging.error(Ta('Failed moving %s to %s'), path, new_path)
-                logging.info("Traceback: ", exc_info = True)
+                if not (cfg.marker_file() and cfg.marker_file() in path):
+                    logging.error(Ta('Failed moving %s to %s'), path, new_path)
+                    logging.info("Traceback: ", exc_info = True)
                 new_path = None
     return new_path
 
@@ -767,6 +838,10 @@ def cleanup_empty_directories(path):
                     pass
         if not repeat:
             break
+    try:
+        remove_dir(path)
+    except:
+        pass
 
 
 @synchronized(DIR_LOCK)
@@ -835,7 +910,10 @@ def bad_fetch(nzo, url, msg='', retry=False, content=False):
         retry : make retry link in histort
         content : report in history that cause is a bad NZB file
     """
-    msg = unicoder(msg)
+    if msg:
+        msg = unicoder(msg)
+    else:
+        msg = ''
 
     pp = nzo.pp
     if pp is None:
@@ -853,7 +931,7 @@ def bad_fetch(nzo, url, msg='', retry=False, content=False):
     else:
         script = ''
 
-    nzo.status = 'Failed'
+    nzo.status = Status.FAILED
 
 
     if url:
@@ -879,6 +957,13 @@ def bad_fetch(nzo, url, msg='', retry=False, content=False):
     else:
         nzo.fail_msg = msg
 
+    if isinstance(url, int) or url.isdigit():
+        url = 'Newzbin #%s' % url
+    growler.send_notification(T('URL Fetching failed; %s') % '', '%s\n%s' % (msg, url), 'other')
+    if cfg.email_endjob() > 0:
+        #import sabnzbd.emailer
+        sabnzbd.emailer.badfetch_mail(msg, url)
+
     from sabnzbd.nzbqueue import NzbQueue
     assert isinstance(NzbQueue.do, NzbQueue)
     NzbQueue.do.remove(nzo.nzo_id, add_to_history=True)
@@ -889,10 +974,13 @@ def on_cleanup_list(filename, skip_nzb=False):
     """
     lst = cfg.cleanup_list()
     if lst:
-        ext = os.path.splitext(filename)[1].strip().strip('.').lower()
+        name, ext = os.path.splitext(filename)
+        ext = ext.strip().lower()
+        name = name.strip()
         for k in lst:
             item = k.strip().strip('.').lower()
-            if item == ext and not (skip_nzb and item == 'nzb'):
+            item = '.' + item
+            if (item == ext or (ext == '' and item == name)) and not (skip_nzb and item == '.nzb'):
                 return True
     return False
 
@@ -1179,6 +1267,81 @@ def is_writable(path):
     else:
         return True
 
-def has_substance(path):
-    """ Return True when file exists and has some content """
-    return os.path.exists(path) and os.stat(path).st_size > 0
+
+def format_source_url(url):
+    """ Format URL suitable for 'Source' stage """
+    if _HAVE_SSL:
+        prot = 'https'
+    else:
+        prot = 'http:'
+    if url and str(url).isdigit():
+        return '%s://%s/browse/post/%s/' % (prot, cfg.newzbin_url(), str(url))
+    else:
+        return url
+
+RE_URL = re.compile(r'://([^/]+)/')
+def get_base_url(url):
+    m = RE_URL.search(url)
+    if m:
+        return m.group(1)
+    else:
+        return ''
+
+def match_str(text, matches):
+    ''' Return first matching element of list 'matches' in 'text', otherwise None '''
+    for match in matches:
+        if match in text:
+            return match
+    return None
+
+def starts_with_path(path, prefix):
+    ''' Return True if 'path' starts with 'prefix',
+        considering case-sensitivity of filesystem
+    '''
+    if sabnzbd.WIN32 or sabnzbd.DARWIN:
+        return path.lower().startswith(prefix.lower())
+    else:
+        return path.startswith(prefix)
+
+
+def set_chmod(path, permissions, report):
+    """ Set 'permissions' on 'path', report any errors when 'report' is True
+    """
+    try:
+        os.chmod(path, permissions)
+    except:
+        lpath = path.lower()
+        if report and '.appledouble' not in lpath and '.ds_store' not in lpath:
+            logging.error(Ta('Cannot change permissions of %s'), path)
+            logging.info("Traceback: ", exc_info = True)
+
+
+def set_permissions(path, recursive=True):
+    """ Give folder tree and its files their proper permissions """
+    if not sabnzbd.WIN32:
+        umask = cfg.umask()
+        try:
+            # Make sure that user R is on
+            umask = int(umask, 8) | int('0400', 8)
+            report = True
+        except ValueError:
+            # No or no valid permissions
+            # Use the effective permissions of the session
+            # Don't report errors (because the system might not support it)
+            umask = int('0777', 8) & (sabnzbd.ORG_UMASK ^ int('0777', 8))
+            report = False
+
+        # Remove X bits for files
+        umask_file = umask & int('7666', 8)
+
+        if os.path.isdir(path):
+            if recursive:
+                # Parse the dir/file tree and set permissions
+                for root, dirs, files in os.walk(path):
+                    set_chmod(root, umask, report)
+                    for name in files:
+                        set_chmod(os.path.join(root, name), umask_file, report)
+            else:
+                set_chmod(path, umask, report)
+        else:
+            set_chmod(path, umask_file, report)
