@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -31,9 +31,11 @@ import sabnzbd
 from sabnzbd.encoding import TRANS, UNTRANS, unicode2local, name_fixer, \
      reliable_unpack_names, unicoder, latin1, platform_encode
 from sabnzbd.utils.rarfile import RarFile, is_rarfile
-from sabnzbd.misc import format_time_string, find_on_path, make_script_path, int_conv
+from sabnzbd.misc import format_time_string, find_on_path, make_script_path, int_conv, \
+                         flag_file
 from sabnzbd.tvsort import SeriesSorter
 import sabnzbd.cfg as cfg
+from constants import Status, QCHECK_FILE
 
 if sabnzbd.WIN32:
     try:
@@ -51,7 +53,7 @@ else:
             return repr(self.parameter)
 
 # Regex globals
-RAR_RE = re.compile(r'\.(?P<ext>part\d*\.rar|rar|s\d\d|r\d\d|\d\d\d)$', re.I)
+RAR_RE = re.compile(r'\.(?P<ext>part\d*\.rar|rar|r\d\d|s\d\d|t\d\d|u\d\d|v\d\d|\d\d\d)$', re.I)
 RAR_RE_V3 = re.compile(r'\.(?P<ext>part\d*)$', re.I)
 
 LOADING_RE = re.compile(r'^Loading "(.+)"')
@@ -126,6 +128,8 @@ def external_processing(extern_proc, complete_dir, filename, msgid, nicename, ca
     command = [str(extern_proc), str(complete_dir), str(filename), \
                str(nicename), str(msgid), str(cat), str(group), str(status)]
 
+    if extern_proc.endswith('.py') and (sabnzbd.WIN32 or not os.access(extern_proc, os.X_OK)):
+        command.insert(0, 'python')
     stup, need_shell, command, creationflags = build_command(command)
     env = fix_env()
 
@@ -252,113 +256,114 @@ def match_ts(file):
         set = ''
     return match, set, num
 
+
+def clean_up_joinables(names):
+    ''' Remove joinable files and their .1 backups
+    '''
+    for name in names:
+        if os.path.exists(name):
+            logging.debug("Deleting %s", name)
+            try:
+                os.remove(name)
+            except:
+                pass
+        name1 = name + ".1"
+        if os.path.exists(name1):
+            logging.debug("Deleting %s", name1)
+            try:
+                os.remove(name1)
+            except:
+                pass
+
+def get_seq_number(name):
+    ''' Return sequence number if name as an int
+    '''
+    head, tail = os.path.splitext(name)
+    if tail == '.ts':
+        match, set, num = match_ts(name)
+    else:
+        num = tail[1:]
+    assert isinstance(num, str)
+    if num.isdigit():
+        return int(num)
+    else:
+        return 0
+
 def file_join(nzo, workdir, workdir_complete, delete, joinables):
     """ Join and joinable files in 'workdir' to 'workdir_complete' and
         when succesful, delete originals
     """
     newfiles = []
+
+    # Create matching sets from the list of files
+    joinable_sets = {}
+    set = match = num = None
+    for joinable in joinables:
+        head, tail = os.path.splitext(joinable)
+        if tail == '.ts':
+            match, set, num = match_ts(joinable)
+        if not set:
+            set = head
+        if set not in joinable_sets:
+            joinable_sets[set] = []
+        joinable_sets[set].append(joinable)
+    logging.debug("joinable_sets: %s", joinable_sets)
+
     try:
-        joinable_sets = {}
-        set = match = num = None
-        for joinable in joinables:
-            head, tail = os.path.splitext(joinable)
-            if tail == '.ts':
-                match, set, num = match_ts(joinable)
-            if not set:
-                set = head
-
-            if set not in joinable_sets:
-                joinable_sets[set] = []
-            joinable_sets[set].append(joinable)
-
-        logging.debug("joinable_sets: %s", joinable_sets)
-
+        # Handle each set
         for joinable_set in joinable_sets:
-            try:
-                expected_size = 0
-                # Make sure there are no missing files in the file sequence
-                # Add 1 to the value before adding to take into account .000
-                for i in xrange(len(joinable_sets[joinable_set])+1):
-                    expected_size += i
-                logging.debug("FJN, expsize: %s", expected_size)
+            current = joinable_sets[joinable_set]
+            joinable_sets[joinable_set].sort()
 
-                # Add together the values of .001 (+1 for .000)
-                # To work out the actual size
-                real_size = 0
-                for joinable in joinable_sets[joinable_set]:
-                    head, tail = os.path.splitext(joinable)
-                    if tail == '.ts':
-                        match, set, num = match_ts(joinable)
-                        real_size += num+1
-                    else:
-                        real_size += int(tail[1:])
-                logging.debug("FJN, realsize: %s", real_size)
+            # If par2 already did the work, just remove the files
+            if os.path.exists(joinable_set):
+                logging.debug("file_join(): Skipping %s, (probably) joined by par2", joinable_set)
+                if delete:
+                    clean_up_joinables(current)
+                # done, go to next set
+                continue
 
-                if real_size != expected_size:
-                    msg = T('Expected size did not equal actual size')
-                    nzo.fail_msg = T('File join of %s failed') % msg
-                    nzo.set_unpack_info('Filejoin', T('[%s] Error "%s" while joining files') % (unicoder(joinable_set), msg))
-                    logging.error(Ta('Error "%s" while running file_join on %s'), msg, latin1(nzo.final_name))
-                else:
-                    joinable_sets[joinable_set].sort()
-                    filename = joinable_set
+            # Prepare joined file
+            filename = joinable_set
+            if workdir_complete:
+                filename = filename.replace(workdir, workdir_complete)
+            logging.debug("file_join(): Assembling %s", filename)
+            joined_file = open(filename, 'ab')
 
-                    # Check if par2 repaired this joinable set
-                    if os.path.exists(filename):
-                        logging.debug("file_join(): Skipping %s, (probably) joined by par2", filename)
-                        if delete:
-                            i = 0
-                            for joinable in joinable_sets[joinable_set]:
-                                if os.path.exists(joinable):
-                                    logging.debug("Deleting %s", joinable)
-                                    try:
-                                        os.remove(joinable)
-                                    except:
-                                        pass
-                                path1 = joinable + ".1"
-                                if os.path.exists(path1):
-                                    logging.debug("Deleting %s", path1)
-                                    try:
-                                        os.remove(path1)
-                                    except:
-                                        pass
-                                i += 1
-                        continue
+            # Join the segments
+            size = len(current)
+            n = get_seq_number(current[0])
+            seq_error = n > 1
+            for joinable in current:
+                if get_seq_number(joinable) != n:
+                    seq_error = True
+                perc = (100.0 / size) * n
+                logging.debug("Processing %s", joinable)
+                nzo.set_action_line(T('Joining'), '%.0f%%' % perc)
+                f = open(joinable, 'rb')
+                joined_file.write(f.read())
+                f.close()
+                if delete:
+                    logging.debug("Deleting %s", joinable)
+                    os.remove(joinable)
+                n += 1
 
-                    if workdir_complete:
-                        filename = filename.replace(workdir, workdir_complete)
+            # Remove any remaining .1 files
+            clean_up_joinables(current)
 
-                    logging.debug("file_join(): Assembling %s", filename)
+            # Finish up
+            joined_file.flush()
+            joined_file.close()
+            newfiles.append(filename)
 
-                    joined_file = open(filename, 'ab')
-
-                    i = 0
-                    for joinable in joinable_sets[joinable_set]:
-                        join_num = len(joinable_sets[joinable_set])
-                        perc = (100.0/join_num)*(i)
-                        logging.debug("Processing %s", joinable)
-                        nzo.set_action_line(T('Joining'), '%.0f%%' % perc)
-                        f = open(joinable, 'rb')
-                        joined_file.write(f.read())
-                        f.close()
-                        i += 1
-                        if delete:
-                            logging.debug("Deleting %s", joinable)
-                            os.remove(joinable)
-
-                    joined_file.flush()
-                    joined_file.close()
-                    msg = T('[%s] Joined %s files') % (unicoder(joinable_set), i)
-                    nzo.set_unpack_info('Filejoin', msg, set=joinable_set)
-                    newfiles.append(joinable_set)
-            except:
-                msg = sys.exc_info()[1]
-                nzo.fail_msg = T('File join of %s failed') % msg
+            if seq_error:
+                msg = T('Incomplete sequence of joinable files')
+                nzo.fail_msg = T('File join of %s failed') % unicoder(joinable_set)
                 nzo.set_unpack_info('Filejoin', T('[%s] Error "%s" while joining files') % (unicoder(joinable_set), msg))
                 logging.error(Ta('Error "%s" while running file_join on %s'), msg, latin1(nzo.final_name))
-                return True, []
-
-        return False, newfiles
+            else:
+                msg = T('[%s] Joined %s files') % (unicoder(joinable_set), size)
+                nzo.set_unpack_info('Filejoin', msg, set=joinable_set)
     except:
         msg = sys.exc_info()[1]
         nzo.fail_msg = T('File join of %s failed') % msg
@@ -366,6 +371,8 @@ def file_join(nzo, workdir, workdir_complete, delete, joinables):
         logging.error(Ta('Error "%s" while running file_join on %s'), msg, latin1(nzo.final_name))
         return True, []
 
+    return False, newfiles
+
 
 #------------------------------------------------------------------------------
 # (Un)Rar Functions
@@ -475,10 +482,17 @@ def rar_extract(rarfile, numrars, one_folder, nzo, setname, extraction_path):
         passwords.insert(0, '')
 
     for password in passwords:
-        if password: logging.debug('Trying unrar with password "%s"', password)
+        if password:
+            logging.debug('Trying unrar with password "%s"', password)
+            msg = T('Trying unrar with password "%s"') % unicoder(password)
+            nzo.fail_msg = msg
+            nzo.set_unpack_info('Unpack', msg)
         fail, new_files, rars = rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path, password)
         if fail != 2:
             break
+
+    if fail == 2:
+        logging.error('%s (%s)', Ta('Unpacking failed, archive requires a password'), latin1(os.path.split(rarfile)[1]))
     return fail, new_files, rars
 
 
@@ -602,7 +616,6 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
             nzo.fail_msg = T('Unpacking failed, archive requires a password')
             msg = ('[%s][%s] '+Ta('Unpacking failed, archive requires a password')) % (setname, latin1(filename))
             nzo.set_unpack_info('Unpack', unicoder(msg), set=setname)
-            logging.error('%s (%s)', Ta('Unpacking failed, archive requires a password'), latin1(filename))
             fail = 2
 
         else:
@@ -757,17 +770,18 @@ def par2_repair(parfile_nzf, nzo, workdir, setname):
     setpars = pars_of_set(workdir, setname)
     result = readd = False
 
-    if cfg.quick_check():
-        nzo.status = 'QuickCheck'
-        nzo.set_action_line(T('Repair'), T('Quick Checking'))
-        result = QuickCheck(setname, nzo)
-        if result:
-            logging.info("Quick-check for %s is OK, skipping repair", setname)
-            nzo.set_unpack_info('Repair', T('[%s] Quick Check OK') % unicoder(setname), set=setname)
-            pars = setpars
+    nzo.status = Status.QUICK_CHECK
+    nzo.set_action_line(T('Repair'), T('Quick Checking'))
+    qc_result = QuickCheck(setname, nzo)
+    if qc_result and cfg.quick_check():
+        logging.info("Quick-check for %s is OK, skipping repair", setname)
+        nzo.set_unpack_info('Repair', T('[%s] Quick Check OK') % unicoder(setname), set=setname)
+        pars = setpars
+        result = True
 
     if not result:
-        nzo.status = 'Repairing'
+        flag_file(workdir, QCHECK_FILE, True)
+        nzo.status = Status.REPAIRING
         result = False
         readd = False
         try:
@@ -787,7 +801,10 @@ def par2_repair(parfile_nzf, nzo, workdir, setname):
                 # Remove this set so we don't try to check it again
                 nzo.remove_parset(parfile_nzf.setname)
             else:
-                logging.info('Par verify failed on %s!', parfile)
+                if qc_result:
+                    logging.warning('Par verify failed on %s, while QuickCheck succeeded!', parfile)
+                else:
+                    logging.info('Par verify failed on %s!', parfile)
 
                 if not readd:
                     # Failed to repair -> remove this set
@@ -860,6 +877,7 @@ def par2_repair(parfile_nzf, nzo, workdir, setname):
     return readd, result
 
 
+_RE_BLOCK_FOUND = re.compile('File: "([^"]+)" - found \d+ of \d+ data blocks from "([^"]+)"')
 def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
     """ Run par2 on par-set """
     if cfg.never_repair():
@@ -869,13 +887,14 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
     retry_classic = False
     used_joinables = []
     #set the current nzo status to "Verifying...". Used in History
-    nzo.status = 'Verifying'
+    nzo.status = Status.VERIFYING
     start = time()
 
     classic = classic or not cfg.par2_multicore()
     logging.debug('Par2-classic = %s', classic)
 
-    if (is_new_partype(nzo, setname) and not classic) or not PAR2C_COMMAND:
+    import sabnzbd.assembler
+    if (sabnzbd.assembler.GetMD5Hashes(parfile)[1] and not classic) or not PAR2C_COMMAND:
         if cfg.par_option():
             command = [str(PAR2_COMMAND), cmd, str(cfg.par_option().strip()), parfile]
         else:
@@ -936,7 +955,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
             if line.startswith('Invalid option specified'):
                 msg = T('[%s] PAR2 received incorrect options, check your Config->Switches settings') % unicoder(setname)
                 nzo.set_unpack_info('Repair', msg, set=setname)
-                nzo.status = 'Failed'
+                nzo.status = Status.FAILED
 
             elif line.startswith('All files are correct'):
                 msg = T('[%s] Verified in %s, all files correct') % (unicoder(setname), format_time_string(time() - start))
@@ -954,7 +973,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                 verified = 1
 
             elif line.startswith('Main packet not found'):
-                ## Initialparfile probaly didn't decode properly,
+                ## Initialparfile probably didn't decode properly,
                 logging.info(Ta('Main packet not found...'))
 
                 extrapars = parfile_nzf.extrapars
@@ -964,7 +983,8 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                 ## Look for the smallest par2file
                 block_table = {}
                 for nzf in extrapars:
-                    block_table[int(nzf.blocks)] = nzf
+                    if not nzf.completed:
+                        block_table[int_conv(nzf.blocks)] = nzf
 
                 if block_table:
                     nzf = block_table[min(block_table.keys())]
@@ -972,6 +992,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                     logging.info("Found new par2file %s", nzf.filename)
 
                     nzo.add_parfile(nzf)
+                    extrapars.remove(nzf)
                     ## mark for readd
                     readd = True
                 else:
@@ -979,7 +1000,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                     nzo.fail_msg = msg
                     msg = u'[%s] %s' % (unicoder(setname), msg)
                     nzo.set_unpack_info('Repair', msg, set=setname)
-                    nzo.status = 'Failed'
+                    nzo.status = Status.FAILED
 
 
             elif line.startswith('You need'):
@@ -1000,7 +1021,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                     if nzf.completed:
                         continue
 
-                    blocks = int(nzf.blocks)
+                    blocks = int_conv(nzf.blocks)
 
                     avail_blocks += blocks
 
@@ -1020,7 +1041,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                     nzo.fail_msg = msg
                     msg = u'[%s] %s' % (unicoder(setname), msg)
                     nzo.set_unpack_info('Repair', msg, set=setname)
-                    nzo.status = 'Failed'
+                    nzo.status = Status.FAILED
                     needed_blocks = avail_blocks
                     force = True
 
@@ -1033,7 +1054,9 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                         extrapar_list = block_table[block_size]
 
                         if extrapar_list:
-                            nzo.add_parfile(extrapar_list.pop())
+                            new_nzf = extrapar_list.pop()
+                            nzo.add_parfile(new_nzf)
+                            extrapars.remove(new_nzf)
                             added_blocks += block_size
 
                         else:
@@ -1044,7 +1067,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
 
                     if not force:
                         msg = T('Fetching %s blocks...') % str(added_blocks)
-                        nzo.status = 'Fetching'
+                        nzo.status = Status.FETCHING
                         nzo.set_action_line(T('Fetching'), msg)
 
                 else:
@@ -1052,7 +1075,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                     nzo.fail_msg = msg
                     msg = u'[%s] %s' % (unicoder(setname), msg)
                     nzo.set_unpack_info('Repair', msg, set=setname)
-                    nzo.status = 'Failed'
+                    nzo.status = Status.FAILED
 
 
             elif line.startswith('Repair is possible'):
@@ -1063,7 +1086,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                 chunks = line.split()
                 per = float(chunks[-1][:-1])
                 nzo.set_action_line(T('Repairing'), '%2d%%' % (per))
-                nzo.status = 'Repairing'
+                nzo.status = Status.REPAIRING
 
             elif line.startswith('Repair complete'):
                 msg = T('[%s] Repaired in %s') % (unicoder(setname), format_time_string(time() - start))
@@ -1077,6 +1100,12 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                     if line.find(os.path.split(jn)[1]) > 0:
                         used_joinables.append(jn)
                         break
+                # Special case of joined RAR files, the "of" and "from" must both be RAR files
+                # This prevents the joined rars files from being seen as an extra rar-set
+                m = _RE_BLOCK_FOUND.search(line)
+                if m and '.rar' in m.group(1).lower() and '.rar' in m.group(2).lower():
+                    workdir = os.path.split(parfile)[0]
+                    used_joinables.append(os.path.join(workdir, m.group(1)))
 
             elif 'Could not write' in line and 'at offset 0:' in line and not classic:
                 # Hit a bug in par2-tbb, retry with par2-classic
@@ -1085,7 +1114,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
             elif not verified:
                 if line.startswith('Verifying source files'):
                     nzo.set_action_line(T('Verifying'), '01/%02d' % verifytotal)
-                    nzo.status = 'Verifying'
+                    nzo.status = Status.VERIFYING
 
                 elif line.startswith('Scanning:'):
                     pass
@@ -1103,7 +1132,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                     if verifytotal == 0 or verifynum < verifytotal:
                         verifynum += 1
                         nzo.set_action_line(T('Verifying'), '%02d/%02d' % (verifynum, verifytotal))
-                        nzo.status = 'Verifying'
+                        nzo.status = Status.VERIFYING
                     datafiles.append(m.group(1))
                     continue
 
@@ -1231,7 +1260,10 @@ def build_filelists(workdir, workdir_complete, check_rar=True):
 
     zips = [f for f in filelist if ZIP_RE.search(f)]
 
-    rars = [f for f in filelist if RAR_RE.search(f) and is_rarfile(f)]
+    if check_rar:
+        rars = [f for f in filelist if RAR_RE.search(f) and is_rarfile(f)]
+    else:
+        rars = [f for f in filelist if RAR_RE.search(f)]
 
     ts = [f for f in filelist if TS_RE.search(f) and f not in joinables]
 
@@ -1285,20 +1317,6 @@ def pars_of_set(wdir, setname):
     return list
 
 
-def is_new_partype(nzo, setname):
-    """ Determine the PAR2 program type, based on the filename encoding """
-    pack = nzo.md5packs.get(setname)
-    if not pack:
-        return True
-    for name in pack.keys():
-        try:
-            name.decode('utf-8')
-        except UnicodeDecodeError:
-            # Now we know it's not pure ASCII or UTF-8
-            return False
-    return True
-
-
 def add_s(i):
     """ Return an "s" when 'i' > 1
     """
@@ -1325,33 +1343,35 @@ def unrar_check(rar):
 def sfv_check(sfv_path):
     """ Verify files using SFV file,
         input: full path of sfv, file are assumed to be relative to sfv
-        returns: True when all files verified OK
+        returns: List of failing files or [] when all is OK
     """
+    failed = []
     try:
         fp = open(sfv_path, 'r')
     except:
         logging.info('Cannot open SFV file %s', sfv_path)
-        return False
+        failed.append(unicoder(sfv_path))
+        return failed
     root = os.path.split(sfv_path)[0]
     status = True
     for line in fp:
         line = line.strip('\n\r ')
         if line[0] != ';':
             x = line.rfind(' ')
-            filename = line[:x].strip()
+            filename = platform_encode(line[:x].strip())
             checksum = line[x:].strip()
-            path = os.path.join(root, platform_encode(filename))
+            path = os.path.join(root, filename)
             if os.path.exists(path):
                 if crc_check(path, checksum):
                     logging.debug('File %s passed SFV check', path)
                 else:
-                    logging.warning('File %s did not pass SFV check', latin1(path))
-                    status = False
+                    logging.info('File %s did not pass SFV check', latin1(path))
+                    failed.append(unicoder(filename))
             else:
-                logging.warning('File %s missing in SFV check', latin1(path))
-                status = False
+                logging.info('File %s missing in SFV check', latin1(path))
+                failed.append(unicoder(filename))
     fp.close()
-    return status
+    return failed
 
 
 def crc_check(path, target_crc):
@@ -1375,7 +1395,7 @@ def crc_check(path, target_crc):
 
 def analyse_show(name):
     """ Do a quick SeasonSort check and return basic facts """
-    job = SeriesSorter(name, None, None, force=True)
+    job = SeriesSorter(name, None, None)
     if job.is_match():
         job.get_values()
     info = job.show_info
