@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,19 +27,20 @@ import datetime
 import sabnzbd
 from sabnzbd.trylist import TryList
 from sabnzbd.nzbstuff import NzbObject
-from sabnzbd.misc import exit_sab, cat_to_opts, \
+from sabnzbd.misc import exit_sab, cat_to_opts, flag_file, \
                          get_admin_path, remove_all, globber
 from sabnzbd.panic import panic_queue
 import sabnzbd.database as database
 from sabnzbd.decorators import NZBQUEUE_LOCK, synchronized, synchronized_CV
 from sabnzbd.constants import QUEUE_FILE_NAME, QUEUE_VERSION, FUTURE_Q_FOLDER, JOB_ADMIN, \
                               LOW_PRIORITY, NORMAL_PRIORITY, HIGH_PRIORITY, TOP_PRIORITY, \
-                              REPAIR_PRIORITY, PNFO_BYTES_FIELD, PNFO_BYTES_LEFT_FIELD
+                              REPAIR_PRIORITY, STOP_PRIORITY, VERIFIED_FILE, \
+                              PNFO_BYTES_FIELD, PNFO_BYTES_LEFT_FIELD, Status
 import sabnzbd.cfg as cfg
 from sabnzbd.articlecache import ArticleCache
 import sabnzbd.downloader
 from sabnzbd.assembler import Assembler, file_has_articles
-from sabnzbd.utils import osx
+import sabnzbd.growler as growler
 from sabnzbd.encoding import latin1, platform_encode
 from sabnzbd.bpsmeter import BPSMeter
 
@@ -130,7 +131,7 @@ class NzbQueue(TryList):
         items = sabnzbd.proxy_build_history()[0]
         # Anything waiting or active or retryable is a known item
         registered.extend([platform_encode(os.path.basename(item['path'])) \
-                           for item in items if item['retry'] or item['loaded'] or item['status'] == 'Queued'])
+                           for item in items if item['retry'] or item['loaded'] or item['status'] == Status.QUEUED])
 
         # Repair unregistered folders
         for folder in globber(cfg.download_dir.get_path()):
@@ -149,8 +150,13 @@ class NzbQueue(TryList):
         """ Reconstruct admin for a single job folder, optionally with new NZB """
         name = os.path.basename(folder)
         path = os.path.join(folder, JOB_ADMIN)
-        if new_nzb is None or not new_nzb.filename:
-            filename = globber(path, '*.gz')
+        if hasattr(new_nzb, 'filename'):
+            filename = new_nzb.filename
+        else:
+            filename = ''
+        if not filename:
+            if not flag_file(folder, VERIFIED_FILE):
+                filename = globber(path, '*.gz')
             if len(filename) > 0:
                 logging.debug('Repair job %s by reparsing stored NZB', latin1(name))
                 sabnzbd.add_nzbfile(filename[0], pp=None, script=None, cat=None, priority=None, nzbname=name, reuse=True)
@@ -160,7 +166,7 @@ class NzbQueue(TryList):
                 self.add(nzo)
         else:
             remove_all(path, '*.gz')
-            logging.debug('Repair job %s without new NZB (%s)', latin1(name), latin1(new_nzb.filename))
+            logging.debug('Repair job %s with new NZB (%s)', latin1(name), latin1(filename))
             sabnzbd.add_nzbfile(new_nzb, pp=None, script=None, cat=None, priority=None, nzbname=name, reuse=True)
 
 
@@ -190,7 +196,7 @@ class NzbQueue(TryList):
     @synchronized(NZBQUEUE_LOCK)
     def generate_future(self, msg, pp=None, script=None, cat=None, url=None, priority=NORMAL_PRIORITY, nzbname=None):
         """ Create and return a placeholder nzo object """
-        future_nzo = NzbObject(msg, 0, pp, script, None, True, cat=cat, url=url, priority=priority, nzbname=nzbname, status="Grabbing")
+        future_nzo = NzbObject(msg, 0, pp, script, None, True, cat=cat, url=url, priority=priority, nzbname=nzbname, status=Status.GRABBING)
         self.add(future_nzo)
         return future_nzo
 
@@ -269,7 +275,12 @@ class NzbQueue(TryList):
     @synchronized(NZBQUEUE_LOCK)
     def change_name(self, nzo_id, name):
         if nzo_id in self.__nzo_table:
-            self.__nzo_table[nzo_id].set_final_name_pw(name)
+            nzo = self.__nzo_table[nzo_id]
+            if not nzo.futuretype:
+                nzo.set_final_name_pw(name)
+            else:
+                # Reset url fetch wait time
+                nzo.wait = None
 
     @synchronized(NZBQUEUE_LOCK)
     def get_nzo(self, nzo_id):
@@ -288,7 +299,7 @@ class NzbQueue(TryList):
         if not nzo.files and not nzo.futuretype:
             sabnzbd.remove_data(nzo.nzo_id, nzo.workpath)
             sabnzbd.proxy_postproc(nzo)
-            return
+            return ''
 
         # Reset try_lists
         nzo.reset_try_list()
@@ -328,10 +339,11 @@ class NzbQueue(TryList):
                 self.save(nzo)
 
             if not (quiet or nzo.status in ('Fetching',)):
-                osx.sendGrowlMsg(T('NZB added to queue'), nzo.filename, osx.NOTIFICATION['download'])
+                growler.send_notification(T('NZB added to queue'), nzo.filename, 'download')
 
         if cfg.auto_sort():
             self.sort_by_avg_age()
+        return nzo.nzo_id
 
     @synchronized(NZBQUEUE_LOCK)
     def remove(self, nzo_id, add_to_history = True, save=True, cleanup=True, keep_basic=False, del_files=False):
@@ -533,6 +545,11 @@ class NzbQueue(TryList):
             nzo_id_pos1 = -1
             pos = -1
 
+            # If priority == STOP_PRIORITY, then send to queue
+            if priority == STOP_PRIORITY:
+                self.end_job(nzo)
+                return
+
             # Get the current position in the queue
             for i in xrange(len(self.__nzo_list)):
                 if nzo_id == self.__nzo_list[i].nzo_id:
@@ -544,6 +561,7 @@ class NzbQueue(TryList):
                 return nzo_id_pos1
 
             nzo.priority = priority
+            nzo.save_attribs()
 
             if nzo_id_pos1 != -1:
                 del self.__nzo_list[nzo_id_pos1]
@@ -617,7 +635,7 @@ class NzbQueue(TryList):
         elif self.__top_only:
             for nzo in self.__nzo_list:
                 # Ignore any items that are in a paused or grabbing state
-                if nzo.status not in ('Paused', 'Grabbing'):
+                if nzo.status not in (Status.PAUSED, Status.GRABBING):
                     return not nzo.server_in_try_list(server)
         else:
             return not self.server_in_try_list(server)
@@ -627,7 +645,7 @@ class NzbQueue(TryList):
         ''' Check if the queue contains any Forced
         Priority items to download while paused '''
         for nzo in self.__nzo_list:
-            if nzo.priority == TOP_PRIORITY and nzo.status not in ('Paused', 'Grabbing'):
+            if nzo.priority == TOP_PRIORITY and nzo.status not in (Status.PAUSED, Status.GRABBING):
                 return True
         return False
 
@@ -636,7 +654,7 @@ class NzbQueue(TryList):
         if self.__top_only:
             if self.__nzo_list:
                 for nzo in self.__nzo_list:
-                    if nzo.status not in ('Paused', 'Grabbing'):
+                    if nzo.status not in (Status.PAUSED, Status.GRABBING):
                         article = nzo.get_article(server)
                         if article:
                             return article
@@ -644,7 +662,7 @@ class NzbQueue(TryList):
         else:
             for nzo in self.__nzo_list:
                 # Don't try to get an article if server is in try_list of nzo
-                if not nzo.server_in_try_list(server) and nzo.status not in ('Paused', 'Grabbing'):
+                if not nzo.server_in_try_list(server) and nzo.status not in (Status.PAUSED, Status.GRABBING):
                     article = nzo.get_article(server)
                     if article:
                         return article
@@ -653,15 +671,15 @@ class NzbQueue(TryList):
             self.add_to_try_list(server)
 
     @synchronized(NZBQUEUE_LOCK)
-    def register_article(self, article):
+    def register_article(self, article, found=True):
         nzf = article.nzf
         nzo = nzf.nzo
 
-        if nzo.deleted or nzf.deleted:
+        if nzf.deleted:
             logging.debug("Discarding article %s, no longer in queue", article.article)
             return
 
-        file_done, post_done, reset = nzo.remove_article(article)
+        file_done, post_done, reset = nzo.remove_article(article, found)
 
         filename = nzf.filename
 
@@ -677,26 +695,32 @@ class NzbQueue(TryList):
                 else:
                     nzo.next_save = time.time() + nzo.save_timeout
 
-            _type = nzf.type
+            if not nzo.precheck:
+                _type = nzf.type
 
-            # Only start decoding if we have a filename and type
-            if filename and _type:
-                Assembler.do.process((nzo, nzf))
-
-            else:
-                if file_has_articles(nzf):
-                    logging.warning(Ta('%s -> Unknown encoding'), filename)
+                # Only start decoding if we have a filename and type
+                if filename and _type:
+                    Assembler.do.process((nzo, nzf))
 
+                else:
+                    if file_has_articles(nzf):
+                        logging.warning(Ta('%s -> Unknown encoding'), filename)
         if post_done:
-            self.finish_nzo(nzo)
+            self.end_job(nzo)
 
-    def finish_nzo(self, nzo):
+
+    def end_job(self, nzo):
+        """ Send NZO to the post-processing queue
+        """
         if self.actives(grabs=False) < 2 and cfg.autodisconnect():
             # This was the last job, close server connections
             sabnzbd.downloader.Downloader.do.disconnect()
 
         # Notify assembler to call postprocessor
-        Assembler.do.process((nzo, None))
+        if not nzo.deleted:
+            nzo.deleted = True
+            Assembler.do.process((nzo, None))
+
 
     @synchronized(NZBQUEUE_LOCK)
     def actives(self, grabs=True):
@@ -705,27 +729,42 @@ class NzbQueue(TryList):
         n = 0
         for nzo in self.__nzo_list:
             # Ignore any items that are paused
-            if grabs and nzo.status == 'Grabbing':
+            if grabs and nzo.status == Status.GRABBING:
                 n += 1
-            elif nzo.status not in ('Paused', 'Grabbing'):
+            elif nzo.status not in (Status.PAUSED, Status.GRABBING):
                 n += 1
         return n
 
 
     @synchronized(NZBQUEUE_LOCK)
-    def queue_info(self, for_cli = False):
+    def queue_info(self, for_cli=False, max_jobs=0):
         bytes_left = 0
         bytes = 0
         pnfo_list = []
+        n = 0
         for nzo in self.__nzo_list:
             pnfo = nzo.gather_info(for_cli = for_cli)
             if nzo.status != 'Paused':
                 bytes += pnfo[PNFO_BYTES_FIELD]
                 bytes_left += pnfo[PNFO_BYTES_LEFT_FIELD]
             pnfo_list.append(pnfo)
-
+            n += 1
+            if max_jobs and n >= max_jobs:
+                break
         return (bytes, bytes_left, pnfo_list)
 
+
+    @synchronized(NZBQUEUE_LOCK)
+    def remaining(self):
+        """ Return bytes left in the queue by non-paused items
+        """
+        bytes_left = 0
+        for nzo in self.__nzo_list:
+            if nzo.status != 'Paused':
+                bytes_left += nzo.remaining()
+        return bytes_left
+
+
     @synchronized(NZBQUEUE_LOCK)
     def is_empty(self):
         empty = True
@@ -820,7 +859,7 @@ def sort_queue_function(nzo_list, method, reverse):
 
 @synchronized_CV
 def add_nzo(nzo):
-    NzbQueue.do.add(nzo)
+    return NzbQueue.do.add(nzo)
 
 @synchronized_CV
 def insert_future_nzo(future_nzo, filename, msgid, data, pp=None, script=None, cat=None, priority=NORMAL_PRIORITY, nzbname=None, nzo_info=None):
