@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -36,10 +36,12 @@ except ImportError:
 import sabnzbd
 from sabnzbd.constants import sample_match, GIGI, ATTRIB_FILE, JOB_ADMIN, \
                               DEFAULT_PRIORITY, LOW_PRIORITY, NORMAL_PRIORITY, \
-                              HIGH_PRIORITY, PAUSED_PRIORITY, TOP_PRIORITY, DUP_PRIORITY
+                              HIGH_PRIORITY, PAUSED_PRIORITY, TOP_PRIORITY, DUP_PRIORITY, \
+                              Status
 from sabnzbd.misc import to_units, cat_to_opts, cat_convert, sanitize_foldername, \
-                         get_unique_path, get_admin_path, remove_all, \
-                         sanitize_filename, globber, sanitize_foldername, int_conv
+                         get_unique_path, get_admin_path, remove_all, format_source_url, \
+                         sanitize_filename, globber, sanitize_foldername, int_conv, \
+                         set_permissions
 import sabnzbd.cfg as cfg
 from sabnzbd.trylist import TryList
 from sabnzbd.encoding import unicoder, platform_encode, latin1, name_fixer
@@ -49,10 +51,10 @@ __all__ = ['Article', 'NzbFile', 'NzbObject']
 # Name potterns
 RE_NEWZBIN = re.compile(r"msgid_(\w+) (.+)(\.nzb)$", re.I)
 RE_NORMAL  = re.compile(r"(.+)(\.nzb)", re.I)
-SUBJECT_FN_MATCHER = re.compile(r'"(.*)"')
+SUBJECT_FN_MATCHER = re.compile(r'"([^"]*)"')
 RE_SAMPLE = re.compile(sample_match, re.I)
 PROBABLY_PAR2_RE = re.compile(r'(.*)\.vol(\d*)\+(\d*)\.par2', re.I)
-
+REJECT_PAR2_RE = re.compile(r'\.par2\.\d+', re.I) # Reject duplicate par2 files
 
 ################################################################################
 # Article                                                                      #
@@ -79,6 +81,7 @@ class Article(TryList):
         self.art_id = None
         self.bytes = bytes
         self.partnum = partnum
+        self.tries = 0 # Try count
         self.nzf = nzf
 
     def get_article(self, server):
@@ -88,6 +91,8 @@ class Article(TryList):
 
         if not self.fetcher and not self.server_in_try_list(server):
             self.fetcher = server
+            self.tries += 1
+            if sabnzbd.LOG_ALL: logging.debug('Article-try = %s', self.tries)
             return self
         return None
 
@@ -115,6 +120,7 @@ class Article(TryList):
         TryList.__init__(self)
         self.fetcher = None
         self.allow_fill_server = False
+        self.tries = 0
 
     def __repr__(self):
         return "<Article: article=%s, bytes=%s, partnum=%s, art_id=%s>" % \
@@ -194,9 +200,7 @@ class NzbFile(TryList):
         self.valid = bool(article_db)
 
         if self.valid and self.nzf_id:
-            if not sabnzbd.save_data(article_db, self.nzf_id, nzo.workpath):
-                logging.info('Saving %s failed', latin1(os.path.join(nzo.workpath, self.nzf_id)))
-                raise IOError
+            sabnzbd.save_data(article_db, self.nzf_id, nzo.workpath)
 
     def finish_import(self):
         """ Load the article objects from disk """
@@ -217,10 +221,12 @@ class NzbFile(TryList):
             self.initial_article = self.decodetable[self.lowest_partnum]
             self.import_finished = True
 
-    def remove_article(self, article):
+    def remove_article(self, article, found):
         """ Handle completed article, possibly end of file """
-        self.articles.remove(article)
-        self.bytes_left -= article.bytes
+        if article in self.articles:
+            self.articles.remove(article)
+            if found:
+                self.bytes_left -= article.bytes
 
         reset = False
         if article.partnum == self.lowest_partnum and self.articles:
@@ -266,7 +272,7 @@ class NzbFile(TryList):
     @property
     def completed(self):
         """ Is this file completed? """
-        return not bool(self.articles)
+        return self.import_finished and not bool(self.articles)
 
     @property
     def lowest_partnum(self):
@@ -497,7 +503,8 @@ NzbObjectMapper = (
     ('duplicate',                    'duplicate'),     # Was detected as a duplicate
     ('oversized',                    'oversized'),     # Was detected as oversized
     ('create_group_folder',          'create_group_folder'),
-    ('aborted',                      'aborted')        # Download aborted due to admin errors
+    ('precheck',                     'precheck'),
+    ('incomplete',                   'incomplete')     # Was detected as incomplete
 )
 
 class NzbObject(TryList):
@@ -542,10 +549,10 @@ class NzbObject(TryList):
         self.script = script        # External script for this set
         self.msgid = '0'            # Newzbin msgid
         self.cat = cat              # Newzbin category
-        if futuretype:
-            self.url = str(url)     # Either newzbin-id or URL queued (future-type only)
+        if url:
+            self.url = str(url)     # Either newzbin-id or source URL
         else:
-            self.url = ''
+            self.url = filename
         self.groups = []
         self.avg_date = datetime.datetime.fromtimestamp(0.0)
         self.avg_stamp = 0.0        # Avg age in seconds (calculated from avg_age)
@@ -582,7 +589,12 @@ class NzbObject(TryList):
         self.parsed = False
         self.duplicate = False
         self.oversized = False
-        self.aborted = False
+        self.precheck = False
+        self.incomplete = False
+        if self.status == Status.QUEUED and not reuse:
+            self.precheck = cfg.pre_check()
+            if self.precheck:
+                self.status = Status.CHECKING
 
         # Store one line responses for filejoin/par2/unrar/unzip here for history display
         self.action_line = ''
@@ -604,6 +616,7 @@ class NzbObject(TryList):
         self.save_timeout = None
         self.new_caching = True
         self.encrypted = 0
+        self.wait = None
         self.pp_active = False  # Signals active post-processing (not saved)
 
         self.create_group_folder = cfg.create_group_folders()
@@ -637,6 +650,7 @@ class NzbObject(TryList):
             remove_all(adir, 'SABnzbd_article_*')
         else:
             wdir = get_unique_path(wdir, create_dir=True)
+            set_permissions(wdir)
             adir = os.path.join(wdir, JOB_ADMIN)
 
         if not os.path.exists(adir):
@@ -662,25 +676,32 @@ class NzbObject(TryList):
             try:
                 parser.parse(inpsrc)
             except xml.sax.SAXParseException, err:
-                self.purge_data(keep_basic=reuse)
-                logging.warning(Ta('Invalid NZB file %s, skipping (reason=%s, line=%s)'),
-                              filename, err.getMessage(), err.getLineNumber())
-                raise ValueError
-            except IOError:
-                self.purge_data(keep_basic=reuse)
-                logging.warning(Ta('Cannot save NZB admin files for "%s"'), latin1(self.work_name))
-                raise ValueError
+                self.incomplete = True
+                if '</nzb>' not in nzb:
+                    logging.warning(Ta('Incomplete NZB file %s'), filename)
+                else:
+                    logging.warning(Ta('Invalid NZB file %s, skipping (reason=%s, line=%s)'),
+                                    filename, err.getMessage(), err.getLineNumber())
             except Exception, err:
-                self.purge_data(keep_basic=reuse)
+                self.incomplete = True
                 logging.warning(Ta('Invalid NZB file %s, skipping (reason=%s, line=%s)'), filename, err, 0)
-                raise ValueError
+
+            if self.incomplete:
+                if cfg.allow_incomplete_nzb():
+                    self.pause()
+                else:
+                    self.purge_data()
+                    raise ValueError
 
             sabnzbd.backup_nzb(filename, nzb)
             sabnzbd.save_compressed(adir, filename, nzb)
 
         if not self.files and not reuse:
             self.purge_data(keep_basic=False)
-            logging.warning(Ta('Empty NZB file %s'), filename)
+            if self.url:
+                logging.warning(Ta('Empty NZB file %s') + ' [%s]', filename, self.url)
+            else:
+                logging.warning(Ta('Empty NZB file %s'), filename)
             raise ValueError
 
         if cat is None:
@@ -694,7 +715,7 @@ class NzbObject(TryList):
 
         # Pickup backed-up attributes when re-using
         if reuse:
-            cat, pp, script, self.priority, name = get_attrib_file(self.workpath, 5)
+            cat, pp, script, priority, name, self.url = get_attrib_file(self.workpath, 6)
             self.set_final_name_pw(name)
 
         # Determine category and find pp/script values
@@ -704,7 +725,7 @@ class NzbObject(TryList):
         # Run user pre-queue script if needed
         if not reuse:
             accept, name, pp, cat, script, priority, group = \
-                    sabnzbd.proxy_pre_queue(self.final_name_pw, pp, cat, script,
+                    sabnzbd.proxy_pre_queue(self.final_name_pw_clean, pp, cat, script,
                                             priority, self.bytes, self.groups)
             accept = int_conv(accept)
             try:
@@ -731,8 +752,8 @@ class NzbObject(TryList):
 
 
         # Pause job when above size limit
-        limit = cfg.SIZE_LIMIT.get_int()
-        if not reuse and limit and self.bytes > limit:
+        limit = cfg.size_limit.get_int()
+        if not reuse and abs(limit) > 0.5 and self.bytes > limit:
             logging.info('Job too large, forcing low prio and paused (%s)', self.work_name)
             self.pause()
             self.oversized = True
@@ -789,6 +810,7 @@ class NzbObject(TryList):
         if nzf in self.files:
             self.files.remove(nzf)
             self.finished_files.append(nzf)
+            nzf.import_finished = True
             nzf.deleted = True
         return not bool(self.files)
 
@@ -797,19 +819,36 @@ class NzbObject(TryList):
             nzf.reset_all_try_lists()
         self.reset_try_list()
 
+    def postpone_pars(self, nzf, parset):
+        """ Move all vol-par files matching 'parset' to the extrapars table """
+        self.partable[parset] = nzf
+        self.extrapars[parset] = []
+        nzf.extrapars = self.extrapars[parset]
+        lparset = parset.lower()
+        for xnzf in self.files[:]:
+            name = xnzf.filename or platform_encode(xnzf.subject)
+            # Move only when not current NZF and filename was extractable from subject
+            if name and nzf is not xnzf:
+                head, vol, block = analyse_par2(name)
+                # When only subject is known, it's enough that that 'parset' is in subject
+                if head and lparset in head.lower():
+                    xnzf.set_par2(parset, vol, block)
+                    self.extrapars[parset].append(xnzf)
+                    self.files.remove(xnzf)
 
     def handle_par2(self, nzf, file_done):
-        ## Special treatment for first part of par2 file
+        """ Check if file is a par2 and build up par2 collection
+        """
         fn = nzf.filename
-        if (not nzf.is_par2) and fn and fn.strip().lower().endswith('.par2'):
-            if fn:
-                par2match = re.search(PROBABLY_PAR2_RE, fn)
+        if fn:
+            # We have a real filename now
+            fn = fn.strip()
+            lfn = fn.lower()
+            if not nzf.is_par2:
+                head, vol, block = analyse_par2(fn)
                 ## Is a par2file and repair mode activated
-                if par2match and (self.repair or cfg.allow_streaming()):
-                    head = par2match.group(1)
-                    nzf.set_par2(par2match.group(1),
-                                par2match.group(2),
-                                par2match.group(3))
+                if head and (self.repair or cfg.allow_streaming()):
+                    nzf.set_par2(head, vol, block)
                     ## Already got a parfile for this set?
                     if head in self.partable:
                         nzf.extrapars = self.extrapars[head]
@@ -826,28 +865,28 @@ class NzbObject(TryList):
                         ## This file either has more blocks,
                         ## or initialparfile is already decoded
                         else:
-                            if not file_done:
-                                nzf.reset_try_list()
+                            if file_done:
                                 if nzf in self.files: self.files.remove(nzf)
-                                self.extrapars[head].append(nzf)
+                                if nzf not in self.extrapars[head]: self.extrapars[head].append(nzf)
+                            else:
+                                nzf.reset_try_list()
+
                     ## No par2file in this set yet, set this as
                     ## initialparfile
                     else:
-                        self.partable[head] = nzf
-                        self.extrapars[head] = []
-                        nzf.extrapars = self.extrapars[head]
+                        self.postpone_pars(nzf, head)
                 ## Is not a par2file or nothing todo
                 else:
                     pass
-            ## No filename in seg 1? Probably not uu or yenc encoded
-            ## Set subject as filename
-            else:
-                nzf.filename = nzf.subject
+        ## No filename in seg 1? Probably not uu or yenc encoded
+        ## Set subject as filename
+        else:
+            nzf.filename = nzf.subject
 
 
-    def remove_article(self, article):
+    def remove_article(self, article, found):
         nzf = article.nzf
-        file_done, reset = nzf.remove_article(article)
+        file_done, reset = nzf.remove_article(article, found)
 
         if file_done:
             self.remove_nzf(nzf)
@@ -855,13 +894,14 @@ class NzbObject(TryList):
         if reset:
             self.reset_try_list()
 
-        self.handle_par2(nzf, file_done)
+        if file_done:
+            self.handle_par2(nzf, file_done)
 
         post_done = False
         if not self.files:
             post_done = True
             #set the nzo status to return "Queued"
-            self.status = 'Queued'
+            self.status = Status.QUEUED
             self.set_download_report()
 
         return (file_done, post_done, reset)
@@ -885,6 +925,7 @@ class NzbObject(TryList):
                 subject = sanitize_filename(latin1(nzf.subject))
                 if (nzf.filename == filename) or (subject == filename) or (filename in subject):
                     nzf.filename = filename
+                    nzf.completed = True
                     self.handle_par2(nzf, file_done=True)
                     self.remove_nzf(nzf)
                     nzfs.remove(nzf)
@@ -927,6 +968,12 @@ class NzbObject(TryList):
             prefix += Ta('ENCRYPTED') + ' / '  #: Queue indicator for encrypted job
         if self.oversized and self.status == 'Paused':
             prefix += Ta('TOO LARGE') + ' / ' #: Queue indicator for oversized job
+        if self.incomplete and self.status == 'Paused':
+            prefix += Ta('INCOMPLETE') + ' / ' #: Queue indicator for incomplete NZB
+        if isinstance(self.wait, float):
+            dif = int(self.wait - time.time() + 0.5)
+            if dif > 0:
+                prefix += Ta('WAIT %s sec') % dif + ' / ' #: Queue indicator for waiting URL fetch
         if self.password:
             return '%s%s / %s' % (name_fixer(prefix), self.final_name, self.password)
         else:
@@ -952,21 +999,52 @@ class NzbObject(TryList):
             sabnzbd.save_data(self, self.nzo_id, self.workpath)
 
     def resume(self):
-        self.status = 'Queued'
+        self.status = Status.QUEUED
         if self.encrypted:
             # If user resumes after encryption warning, no more auto-pauses
             self.encrypted = 2
-        # If user resumes after warning, reset duplicate/oversized indicator
+        # If user resumes after warning, reset duplicate/oversized/incomplete indicators
         self.duplicate = False
         self.oversized = False
+        self.incomplete = False
 
     def add_parfile(self, parfile):
-        self.files.append(parfile)
-        parfile.extrapars.remove(parfile)
+        if parfile not in self.files:
+            self.files.append(parfile)
+        if parfile.extrapars and parfile in parfile.extrapars:
+            parfile.extrapars.remove(parfile)
 
     def remove_parset(self, setname):
         self.partable.pop(setname)
 
+    __re_quick_par2_check = re.compile('\.par2\W*', re.I)
+    def check_quality(self):
+        """ Determine amount of articles present on servers
+            and return (gross available, nett) bytes
+        """
+        need = 0L
+        pars = 0L
+        short = 0L
+        anypars = False
+        for nzf_id in self.files_table:
+            nzf = self.files_table[nzf_id]
+            assert isinstance(nzf, NzbFile)
+            short += nzf.bytes_left
+            if self.__re_quick_par2_check.search(nzf.subject):
+                pars += nzf.bytes
+                anypars = True
+            else:
+                need += nzf.bytes
+        have = need + pars - short
+        ratio = float(have) / float(max(1, need))
+        if anypars:
+            enough = ratio * 100.0 >= float(cfg.req_completion_rate())
+        else:
+            enough = have >= need
+        logging.debug('Download Quality: enough=%s, have=%s, need=%s, ratio=%s', enough, have, need, ratio)
+        return enough, ratio
+
+
     def set_download_report(self):
         if self.avg_bps_total and self.bytes_downloaded and self.avg_bps_freq:
             #get the deltatime since the download started
@@ -994,6 +1072,8 @@ class NzbObject(TryList):
                 msg4 = ('<br/>' + T('%s articles had non-matching duplicates')) % len(dups)
             msg = ''.join((msg1, msg2, msg3, msg4,))
             self.set_unpack_info('Download', msg, unique=True)
+            if self.url:
+                self.set_unpack_info('Source', format_source_url(self.url), unique=True)
 
     def inc_log(self, log, txt):
         """ Append string txt to nzo_info element "log"
@@ -1023,9 +1103,7 @@ class NzbObject(TryList):
                             if not nzf.import_finished:
                                 logging.error(Ta('Error importing %s'), nzf)
                                 nzf_remove_list.append(nzf)
-                                nzf.nzo.aborted = True
-                                sabnzbd.nzbqueue.NzbQueue.do.finish_nzo(nzf.nzo)
-                                return None
+                                continue
                         else:
                             continue
 
@@ -1033,8 +1111,14 @@ class NzbObject(TryList):
                     if article:
                         break
 
+        # Remove all files for which admin could not be read
         for nzf in nzf_remove_list:
+            nzf.deleted = True
+            nzf.completed = True
             self.files.remove(nzf)
+        # If cleanup emptied the active files list, end this job
+        if nzf_remove_list and not self.files:
+            sabnzbd.NzbQueue.do.end_job(self)
 
         if not article:
             # No articles for this server, block for next time
@@ -1151,6 +1235,13 @@ class NzbObject(TryList):
                 except:
                     pass
 
+    def remaining(self):
+        """ Return remaining bytes """
+        bytes_left = 0
+        for nzf in self.files:
+            bytes_left += nzf.bytes_left
+        return bytes_left
+
     def gather_info(self, for_cli = False):
         bytes_left_all = 0
 
@@ -1245,11 +1336,7 @@ class NzbObject(TryList):
         return self.repair, self.unpack, self.delete
 
     def save_attribs(self):
-        if self.priority == TOP_PRIORITY:
-            prio = HIGH_PRIORITY
-        else:
-            prio = self.priority
-        set_attrib_file(self.workpath, (self.cat, self.pp, self.script, prio, self.final_name_pw))
+        set_attrib_file(self.workpath, (self.cat, self.pp, self.script, self.priority, self.final_name_pw_clean, self.url))
 
     def build_pos_nzf_table(self, nzf_ids):
         pos_nzf_table = {}
@@ -1286,6 +1373,7 @@ class NzbObject(TryList):
                 self.__dict__[tup[1]] = None
         self.pp_active = False
         self.avg_stamp = time.mktime(self.avg_date.timetuple())
+        self.wait = None
         TryList.__init__(self)
 
 
@@ -1311,7 +1399,7 @@ def nzf_cmp_date(nzf1, nzf2):
     return nzf_cmp_name(nzf1, nzf2, name=False)
 
 
-RE_RAR = re.compile(r'(\.rar|\.r\d\d)', re.I)
+RE_RAR = re.compile(r'(\.rar|\.r\d\d|\.s\d\d|\.t\d\d|\.u\d\d|\.v\d\d)$', re.I)
 
 def nzf_cmp_name(nzf1, nzf2, name=True):
     # The comparison will sort .par2 files to the top of the queue followed by .rar files,
@@ -1319,14 +1407,22 @@ def nzf_cmp_name(nzf1, nzf2, name=True):
     name1 = nzf_get_filename(nzf1)
     name2 = nzf_get_filename(nzf2)
 
-    # vol-pars go in front
+    # Determine vol-pars
     is_par1 = '.vol' in name1 and '.par2' in name1
     is_par2 = '.vol' in name2 and '.par2' in name2
-    if is_par1 and not is_par2:
+
+    # mini-par2 in front
+    if not is_par1 and name1.endswith('.par2'):
         return -1
-    if is_par2 and not is_par1:
+    if not is_par2 and name2.endswith('.par2'):
         return 1
 
+    # vol-pars go to the back
+    if is_par1 and not is_par2:
+        return 1
+    if is_par2 and not is_par1:
+        return -1
+
     if name:
         # Prioritise .rar files above any other type of file (other than vol-par)
         # Useful for nzb streaming
@@ -1449,3 +1545,23 @@ def set_attrib_file(path, attribs):
     for item in attribs:
         f.write('%s\n' % item)
     f.close()
+
+
+def analyse_par2(name):
+    """ Check if file is a par2-file and determine vol/block
+        return head, vol, block
+        head is empty when not a par2 file
+    """
+    head = None
+    vol = block = 0
+    if name and not REJECT_PAR2_RE.search(name):
+        m = PROBABLY_PAR2_RE.search(name)
+        if m:
+            head = m.group(1)
+            vol = m.group(2)
+            block = m.group(3)
+        elif name.lower().find('.par2') > 0:
+            head = os.path.splitext(name)[0]
+        else:
+            head = None
+    return head, vol, block
