@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,25 +23,27 @@ from Foundation import *
 from AppKit import *
 from PyObjCTools import AppHelper
 from objc import YES, NO, nil
-from threading import Thread
 
 import os
+import subprocess
+from threading import Thread
+
 import cherrypy
 import Cheetah.DummyTransaction
 import sys
 import time
 
 import logging
-import logging.handlers
 
 import sabnzbd
 import sabnzbd.cfg
 
 from sabnzbd.constants import *
-from sabnzbd.misc import get_filename, get_ext, diskfree
+from sabnzbd.misc import get_filename, get_ext, diskfree, to_units
 from sabnzbd.panic import launch_a_browser
-from sabnzbd.utils import osx
+import sabnzbd.growler as growler
 
+from sabnzbd.api import fast_queue
 from sabnzbd.nzbqueue import NzbQueue
 import sabnzbd.config as config
 import sabnzbd.scheduler as scheduler
@@ -50,7 +52,7 @@ import sabnzbd.dirscanner as dirscanner
 from sabnzbd.bpsmeter import BPSMeter
 from sabnzbd.newzbin import Bookmarks
 from sabnzbd.database import get_history_handle
-from sabnzbd.api import check_trans
+from sabnzbd.encoding import unicoder
 
 status_icons = {'idle':'../Resources/sab_idle.png','pause':'../Resources/sab_pause.png','clicked':'../Resources/sab_clicked.png'}
 start_time = NSDate.date()
@@ -61,6 +63,7 @@ class SABnzbdDelegate(NSObject):
 
     icons = {}
     status_bar = None
+    osx_icon = True
 
     def awakeFromNib(self):
         #Status Bar iniatilize
@@ -92,7 +95,7 @@ class SABnzbdDelegate(NSObject):
         #cherrypy.engine.wait(cherrypy.process.wspbus.states.STARTED)
 
         # Wait for translated texts to be loaded
-        while not check_trans():
+        while not sabnzbd.WEBUI_READY and not sabnzbd.SABSTOP:
             time.sleep(0.5)
             if (debug == 1) : NSLog("[osx] language file not loaded, waiting")
 
@@ -298,12 +301,9 @@ class SABnzbdDelegate(NSObject):
 
     def updateAction_(self, notification):
         try:
-            if os.path.exists("%s/notDisplayMenu" % (sabnzbd.DIR_APPDATA)):
-                sabnzbd.OSX_ICON = 0
-            else:
-                sabnzbd.OSX_ICON = 1
+            self.osx_icon = sabnzbd.cfg.osx_menu()
 
-            if sabnzbd.OSX_ICON:
+            if self.osx_icon:
                 if self.status_removed == 1:
                     self.buildMenu()
 
@@ -331,7 +331,7 @@ class SABnzbdDelegate(NSObject):
 
     def queueUpdate(self):
         try:
-            qnfo = NzbQueue.do.queue_info()
+            qnfo = NzbQueue.do.queue_info(max_jobs=10)
             pnfo_list = qnfo[QNFO_PNFO_LIST_FIELD]
 
             bytesleftprogess = 0
@@ -350,7 +350,7 @@ class SABnzbdDelegate(NSObject):
                 for pnfo in pnfo_list:
                     if job_nb > 10:
                         break
-                    filename = pnfo[PNFO_FILENAME_FIELD]
+                    filename = unicoder(pnfo[PNFO_FILENAME_FIELD])
                     msgid = pnfo[PNFO_MSGID_FIELD]
                     bytesleft = pnfo[PNFO_BYTES_LEFT_FIELD] / MEBI
                     bytesleftprogess += pnfo[PNFO_BYTES_LEFT_FIELD]
@@ -401,7 +401,7 @@ class SABnzbdDelegate(NSObject):
                         menu_history_item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(job, 'openFolderAction:', '')
                     else:
                         menu_history_item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(job, '', '')
-                    if history['status'] != "Completed":
+                    if history['status'] != Status.COMPLETED:
                         jobfailed = NSAttributedString.alloc().initWithString_attributes_(job, self.failedAttributes)
                         menu_history_item.setAttributedTitle_(jobfailed)
                     menu_history_item.setRepresentedObject_("%s" % (path))
@@ -441,26 +441,28 @@ class SABnzbdDelegate(NSObject):
 
     def stateUpdate(self):
         try:
-            qnfo = NzbQueue.do.queue_info()
-            bpsnow = BPSMeter.do.get_bps()
-            if sabnzbd.downloader.Downloader.do.paused:
+            paused, bytes_left, bpsnow, time_left = fast_queue()
+
+            if paused:
                 self.state = T('Paused')
                 if sabnzbd.scheduler.pause_int() != "0":
                     self.setMenuTitle("\n\n%s\n" % (sabnzbd.scheduler.pause_int()))
                 else:
                     self.setMenuTitle("")
-            elif qnfo[QNFO_BYTES_LEFT_FIELD] / MEBI > 0:
-
+            elif bytes_left > 0:
                 self.state = ""
-                speed = self.to_units(bpsnow)
-                timeleft = (speed>1 and self.calc_timeleft(qnfo[QNFO_BYTES_LEFT_FIELD],bpsnow)) or "--"
+                speed = to_units(bpsnow, dec_limit=1)
+                # "10.1 MB/s" doesn't fit, remove space char
+                if 'M' in speed and len(speed) > 5:
+                    speed = speed.replace(' ', '')
+                time_left = (bpsnow>10 and time_left) or "------"
 
-                statusbarText = "\n\n%s\n%s\n" % (timeleft, speed)
+                statusbarText = "\n\n%s\n%sB/s\n" % (time_left, speed)
 
                 if sabnzbd.SABSTOP:
                     statusbarText = "..."
 
-                if os.path.exists("%s/notDisplaySpeed" % (sabnzbd.DIR_APPDATA)):
+                if not sabnzbd.cfg.osx_speed():
                     statusbarText = ""
 
                 self.setMenuTitle(statusbarText)
@@ -526,7 +528,7 @@ class SABnzbdDelegate(NSObject):
             if sabnzbd.NEW_VERSION and self.version_notify:
                 #logging.info("[osx] New Version : %s" % (sabnzbd.NEW_VERSION))
                 new_release, new_rel_url = sabnzbd.NEW_VERSION.split(';')
-                osx.sendGrowlMsg("SABnzbd","%s : %s" % (T('New release available'),new_release),osx.NOTIFICATION['other'])
+                growler.send_notification("SABnzbd","%s : %s" % (T('New release available'), new_release), 'other')
                 self.version_notify = 0
         except :
             logging.info("[osx] versionUpdate Exception %s" % (sys.exc_info()[0]))
@@ -663,28 +665,6 @@ class SABnzbdDelegate(NSObject):
         except:
             return '0:00:00'
 
-    def to_units(self, val):
-        """ Convert number to K/M/G/T/P notation
-        """
-        TAB_UNITS = ("", "K", "M", "G", "T", "P")
-        val = str(val).strip()
-        if val == "-1":
-            return val
-        n=0
-        try:
-            val = float(val)
-        except:
-            return ''
-
-        while (val > 1023.0) and (n < 5):
-            val = val / 1024.0
-            n= n+1
-        unit = TAB_UNITS[n]
-        if n>1:
-            return " %.2f %sB/s " % (val, unit)
-        else:
-            return " %d %sB/s " % (val, unit)
-
     def openBrowserAction_(self, sender):
         if sender.representedObject:
             link = sender.representedObject()
@@ -783,12 +763,16 @@ class SABnzbdDelegate(NSObject):
         logging.info('[osx] application terminating')
         self.setMenuTitle("\n\n%s\n"% (T('Stopping...')))
         self.status_item.setHighlightMode_(NO)
-        sabnzbd.OSX_ICON = 0
+        self.osx_icon = False
         logging.info('[osx] application stopping daemon')
         sabnzbd.halt()
         cherrypy.engine.exit()
         sabnzbd.SABSTOP = True
-        osx.sendGrowlMsg('SABnzbd',T('SABnzbd shutdown finished'),osx.NOTIFICATION['other'])
+        try:
+            growler.send_notification('SABnzbd', T('SABnzbd shutdown finished'), growler.NOTIFICATION['other'])
+        except AttributeError:
+            # Fails for the OSX binary
+            pass
         logging.info('Leaving SABnzbd')
         sys.stderr.flush()
         sys.stdout.flush()
@@ -804,3 +788,40 @@ def notify(notificationName, message):
         nc.postNotificationName_object_(notificationName, message)
         del pool
 
+
+#------------------------------------------------------------------------------
+class OsxAwake(Thread):
+    """ Keep running 'caffeinate' as long as the 'stay_awake' flag is set
+    """
+    do = None
+    def __init__(self):
+        Thread.__init__(self)
+        self.stay_awake = False
+        self.__stop_now = False
+        self.__proc = None
+        logging.info('Starting caffeinate task')
+        OsxAwake.do = self
+
+    def run(self):
+        command = ['caffeinate', '-i', '-s', '-t300']
+        while not self.__stop_now:
+            if self.stay_awake:
+                self.stay_awake = False
+                logging.debug('Launching caffeinate')
+                self.__proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
+                self.__proc.wait()
+                self.__proc = None
+            else:
+                time.sleep(30)
+
+    def stop(self):
+        logging.info('Stopping caffeinate task')
+        self.__stop_now = True
+        self.stay_awake = False
+        if self.__proc:
+            logging.debug('Stopping caffeinate')
+            try:
+                self.__proc.terminate()
+            except:
+                pass
+
