@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -31,11 +31,12 @@ import re
 from sabnzbd.newsunpack import unpack_magic, par2_repair, external_processing, sfv_check
 from threading import Thread
 from sabnzbd.misc import real_path, get_unique_path, create_dirs, move_to_path, \
-                         get_unique_filename, make_script_path, \
-                         on_cleanup_list, renamer, remove_dir, remove_all, globber
+                         get_unique_filename, make_script_path, flag_file, \
+                         on_cleanup_list, renamer, remove_dir, remove_all, globber, \
+                         set_permissions
 from sabnzbd.tvsort import Sorter
 from sabnzbd.constants import REPAIR_PRIORITY, POSTPROC_QUEUE_FILE_NAME, \
-     POSTPROC_QUEUE_VERSION, sample_match, JOB_ADMIN
+     POSTPROC_QUEUE_VERSION, sample_match, JOB_ADMIN, Status, VERIFIED_FILE
 from sabnzbd.encoding import TRANS, unicoder
 from sabnzbd.newzbin import Bookmarks
 import sabnzbd.emailer as emailer
@@ -45,7 +46,7 @@ import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 import sabnzbd.nzbqueue
 import sabnzbd.database as database
-from sabnzbd.utils import osx
+import sabnzbd.growler as growler
 
 
 #------------------------------------------------------------------------------
@@ -224,17 +225,27 @@ def process_job(nzo):
     filename = nzo.final_name
     msgid = nzo.msgid
 
+    if nzo.precheck:
+        # Check result
+        enough, ratio = nzo.check_quality()
+        if enough:
+            # Enough data present, do real download
+            workdir = nzo.downpath
+            sabnzbd.nzbqueue.NzbQueue.do.cleanup_nzo(nzo, keep_basic=True)
+            sabnzbd.nzbqueue.NzbQueue.do.repair_job(workdir)
+            return True
+        else:
+            # Not enough data, flag as failed
+            nzo.save_attribs()
+
     if cfg.allow_streaming() and not (flag_repair or flag_unpack or flag_delete):
         # After streaming, force +D
         nzo.set_pp(3)
-        nzo.status = 'Failed'
+        nzo.status = Status.FAILED
         nzo.save_attribs()
         all_ok = False
 
     try:
-        if nzo.aborted:
-            crash_msg = T('Download aborted due to admin errros')
-            raise IOError
 
         # Get the folder containing the download result
         workdir = nzo.downpath
@@ -242,9 +253,20 @@ def process_job(nzo):
 
         # if no files are present (except __admin__), fail the job
         if len(globber(workdir)) < 2:
-            emsg = T('Download failed - Out of your server\'s retention?')
+            if nzo.precheck:
+                req_ratio = float(cfg.req_completion_rate()) / 100.0
+                # Make sure that rounded ratio doesn't equal required ratio
+                # when it is actually below required
+                if (ratio < req_ratio) and (req_ratio - ratio) < 0.001:
+                    ratio = req_ratio - 0.001
+                emsg = '%.1f%%' % (ratio * 100.0)
+                emsg2 = '%.1f%%' % float(cfg.req_completion_rate())
+                emsg = T('Download might fail, only %s of required %s available') % (emsg, emsg2)
+            else:
+                emsg = T('Download failed - Out of your server\'s retention?')
             nzo.fail_msg = emsg
-            nzo.status = 'Failed'
+            nzo.set_unpack_info('Fail', emsg)
+            nzo.status = Status.FAILED
             # do not run unpacking or parity verification
             flag_repair = flag_unpack = False
             par_error = unpack_error = True
@@ -271,6 +293,7 @@ def process_job(nzo):
         # Set complete dir to workdir in case we need to abort
         workdir_complete = workdir
         dirname = nzo.final_name
+        marker_file = None
 
         if all_ok:
             one_folder = False
@@ -288,13 +311,15 @@ def process_job(nzo):
             ## TV/Movie/Date Renaming code part 1 - detect and construct paths
             file_sorter = Sorter(cat)
             complete_dir = file_sorter.detect(dirname, complete_dir)
-            if file_sorter.is_sortfile():
+            if file_sorter.sort_file:
                 one_folder = False
 
             if one_folder:
                 workdir_complete = create_dirs(complete_dir)
             else:
                 workdir_complete = get_unique_path(os.path.join(complete_dir, dirname), create_dir=True)
+                marker_file = set_marker(workdir_complete)
+
             if not workdir_complete or not os.path.exists(workdir_complete):
                 crash_msg = T('Cannot create final folder %s') % unicoder(os.path.join(complete_dir, dirname))
                 raise IOError
@@ -313,7 +338,7 @@ def process_job(nzo):
             if flag_unpack:
                 if all_ok:
                     #set the current nzo status to "Extracting...". Used in History
-                    nzo.status = 'Extracting'
+                    nzo.status = Status.EXTRACTING
                     logging.info("Running unpack_magic on %s", filename)
                     unpack_error, newfiles = unpack_magic(nzo, workdir, tmp_workdir_complete, flag_delete, one_folder, (), (), (), ())
                     logging.info("unpack_magic finished on %s", filename)
@@ -325,7 +350,7 @@ def process_job(nzo):
 
             if all_ok:
                 ## Move any (left-over) files to destination
-                nzo.status = 'Moving'
+                nzo.status = Status.MOVING
                 nzo.set_action_line(T('Moving'), '...')
                 for root, dirs, files in os.walk(workdir):
                     if not root.endswith(JOB_ADMIN):
@@ -333,14 +358,18 @@ def process_job(nzo):
                             path = os.path.join(root, file_)
                             new_path = path.replace(workdir, tmp_workdir_complete)
                             new_path = get_unique_filename(new_path)
+                            newfiles.append(new_path)
                             if not move_to_path(path, new_path, unique=False):
                                 nzo.set_unpack_info('Unpack', T('Failed moving %s to %s') % (unicoder(path), unicoder(new_path)))
                                 all_ok = False
                                 break
 
             ## Set permissions right
-            if not sabnzbd.WIN32:
-                perm_script(tmp_workdir_complete, cfg.umask())
+            set_permissions(tmp_workdir_complete)
+
+            if all_ok and marker_file:
+                del_marker(os.path.join(tmp_workdir_complete, marker_file))
+                remove_from_list(marker_file, newfiles)
 
             if all_ok:
                 ## Remove files matching the cleanup list
@@ -381,19 +410,21 @@ def process_job(nzo):
                 remove_samples(workdir_complete)
 
             ## TV/Movie/Date Renaming code part 2 - rename and move files to parent folder
-            if all_ok:
-                if newfiles and file_sorter.is_sortfile():
+            if all_ok and file_sorter.sort_file:
+                if newfiles:
                     file_sorter.rename(newfiles, workdir_complete)
                     workdir_complete, ok = file_sorter.move(workdir_complete)
-                    if not ok:
-                        nzo.set_unpack_info('Unpack', T('Failed to move files'))
-                        all_ok = False
+                else:
+                    workdir_complete, ok = file_sorter.rename_with_ext(workdir_complete)
+                if not ok:
+                    nzo.set_unpack_info('Unpack', T('Failed to move files'))
+                    all_ok = False
 
             ## Run the user script
             script_path = make_script_path(script)
             if all_ok and (not nzb_list) and script_path:
                 #set the current nzo status to "Ext Script...". Used in History
-                nzo.status = 'Running'
+                nzo.status = Status.RUNNING
                 nzo.set_action_line(T('Running script'), unicoder(script))
                 nzo.set_unpack_info('Script', T('Running user script %s') % unicoder(script), unique=True)
                 script_log, script_ret = external_processing(script_path, workdir_complete, nzo.filename,
@@ -438,16 +469,16 @@ def process_job(nzo):
         ## Remove newzbin bookmark, if any
         if msgid and all_ok:
             Bookmarks.do.del_bookmark(msgid)
-        elif all_ok:
+        elif all_ok and isinstance(nzo.url, str):
             sabnzbd.proxy_rm_bookmark(nzo.url)
 
         ## Show final status in history
         if all_ok:
-            osx.sendGrowlMsg(T('Download Completed'), filename, osx.NOTIFICATION['complete'])
-            nzo.status = 'Completed'
+            growler.send_notification(T('Download Completed'), filename, 'complete')
+            nzo.status = Status.COMPLETED
         else:
-            osx.sendGrowlMsg(T('Download Failed'), filename, osx.NOTIFICATION['complete'])
-            nzo.status = 'Failed'
+            growler.send_notification(T('Download Failed'), filename, 'complete')
+            nzo.status = Status.FAILED
 
     except:
         logging.error(Ta('Post Processing Failed for %s (%s)'), filename, crash_msg)
@@ -455,10 +486,16 @@ def process_job(nzo):
             logging.info("Traceback: ", exc_info = True)
             crash_msg = T('see logfile')
         nzo.fail_msg = T('PostProcessing was aborted (%s)') % unicoder(crash_msg)
-        osx.sendGrowlMsg(T('Download Failed'), filename, osx.NOTIFICATION['complete'])
-        nzo.status = 'Failed'
+        growler.send_notification(T('Download Failed'), filename, 'complete')
+        nzo.status = Status.FAILED
         par_error = True
         all_ok = False
+        info = nzo.unpack_info.copy()
+        info['fail'] = [nzo.fail_msg]
+        if cfg.email_endjob():
+            emailer.endjob(dirname, msgid, cat, all_ok, workdir_complete, nzo.bytes_downloaded,
+                           info, '', '', 0)
+
 
     if all_ok:
         # If the folder only contains one file OR folder, have that as the path
@@ -503,8 +540,9 @@ def process_job(nzo):
 def parring(nzo, workdir):
     """ Perform par processing. Returns: (par_error, re_add)
     """
+    assert isinstance(nzo, sabnzbd.nzbstuff.NzbObject)
     filename = nzo.final_name
-    osx.sendGrowlMsg(T('Post-processing'), nzo.final_name, osx.NOTIFICATION['pp'])
+    growler.send_notification(T('Post-processing'), nzo.final_name, 'pp')
     logging.info('Par2 check starting on %s', filename)
 
     ## Collect the par files
@@ -525,8 +563,7 @@ def parring(nzo, workdir):
             need_re_add, res = par2_repair(parfile_nzf, nzo, workdir, set_)
             if need_re_add:
                 re_add = True
-            else:
-                par_error = par_error or not res
+            par_error = par_error or not res
 
         if re_add:
             logging.info('Readded %s to queue', filename)
@@ -546,8 +583,12 @@ def parring(nzo, workdir):
             par_error = False
             nzo.set_unpack_info('Repair', T('Trying SFV verification'))
             for sfv in sfvs:
-                if not sfv_check(sfv):
-                    nzo.set_unpack_info('Repair', T('Some files failed to verify against "%s"') % unicoder(os.path.basename(sfv)))
+                failed = sfv_check(sfv)
+                if failed:
+                    msg = T('Some files failed to verify against "%s"') % unicoder(os.path.basename(sfv))
+                    msg += '; '
+                    msg += '; '.join(failed)
+                    nzo.set_unpack_info('Repair', msg)
                     par_error = True
             if not par_error:
                 nzo.set_unpack_info('Repair', T('Verified successfully using SFV files'))
@@ -555,47 +596,14 @@ def parring(nzo, workdir):
             logging.info("No par2 sets for %s", filename)
             nzo.set_unpack_info('Repair', T('[%s] No par2 sets') % unicoder(filename))
 
+    if not par_error:
+        flag_file(workdir, VERIFIED_FILE, create=True)
     return par_error, re_add
 
 
 
 #------------------------------------------------------------------------------
 
-def perm_script(wdir, umask):
-    """ Give folder tree and its files their proper permissions """
-    from os.path import join
-
-    try:
-        # Make sure that user R is on
-        umask = int(umask, 8) | int('0400', 8)
-        report_errors = True
-    except ValueError:
-        # No or no valid permissions
-        # Use the effective permissions of the session
-        # Don't report errors (because the system might not support it)
-        umask = int('0777', 8) & (sabnzbd.ORG_UMASK ^ int('0777', 8))
-        report_errors = False
-
-    # Remove X bits for files
-    umask_file = umask & int('7666', 8)
-
-    # Parse the dir/file tree and set permissions
-    for root, dirs, files in os.walk(wdir):
-        try:
-            os.chmod(root, umask)
-        except:
-            if report_errors:
-                logging.error(Ta('Cannot change permissions of %s'), root)
-                logging.info("Traceback: ", exc_info = True)
-        for name in files:
-            try:
-                os.chmod(join(root, name), umask_file)
-            except:
-                if report_errors:
-                    logging.error(Ta('Cannot change permissions of %s'), join(root, name))
-                    logging.info("Traceback: ", exc_info = True)
-
-
 def addPrefixes(path, dirprefix):
     """ Add list of prefixes as sub folders to path
         '/my/path' and ['a', 'b', 'c'] will give '/my/path/a/b/c'
@@ -741,3 +749,40 @@ def collapse_folder(oldpath, newpath):
         remove_dir(orgpath)
     except:
         pass
+
+
+#------------------------------------------------------------------------------
+def set_marker(folder):
+    """ Set marker file and return name """
+    name = cfg.marker_file()
+    if name:
+        path = os.path.join(folder, name)
+        logging.debug('Create marker file %s', path)
+        try:
+            fp = open(path, 'w')
+            fp.close()
+        except:
+            logging.info('Cannot create marker file %s', path)
+            logging.info("Traceback: ", exc_info = True)
+            name = None
+    return name
+
+
+def del_marker(path):
+    """ Remove marker file """
+    if path and os.path.exists(path):
+        logging.debug('Removing marker file %s', path)
+        try:
+            os.remove(path)
+        except:
+            logging.info('Cannot remove marker file %s', path)
+            logging.info("Traceback: ", exc_info = True)
+
+
+def remove_from_list(name, lst):
+    if name:
+        for n in xrange(len(lst)):
+            if lst[n].endswith(name):
+                logging.debug('Popping %s', lst[n])
+                lst.pop(n)
+                return
