@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -125,7 +125,7 @@ def convert_filter(text):
     try:
         return re.compile(txt, re.I)
     except:
-        logging.error(Ta('Could not compile regex: %s'), text)
+        logging.debug('Could not compile regex: %s', text)
         return None
 
 _EXPIRE_SEC = 3*24*3600 # 3 days
@@ -288,6 +288,7 @@ class RSSQueue(object):
         rePPs = []
         rePrios = []
         reScripts = []
+        reEnabled = []
         for filter in feeds.filters():
             reCat = filter[0]
             if defCat in ('', '*'):
@@ -298,6 +299,7 @@ class RSSQueue(object):
             reTypes.append(filter[3])
             regexes.append(convert_filter(filter[4]))
             rePrios.append(filter[5])
+            reEnabled.append(filter[6] != '0')
         regcount = len(regexes)
 
         # Set first if this is the very first scan of this URI
@@ -318,6 +320,7 @@ class RSSQueue(object):
 
         # Read the RSS feed
         if readout:
+            uri = uri.replace(' ', '%20')
             logging.debug("Running feedparser on %s", uri)
             d = feedparser.parse(uri.replace('feed://', 'http://'))
             logging.debug("Done parsing %s", uri)
@@ -396,26 +399,30 @@ class RSSQueue(object):
 
                     # Match against all filters until an postive or negative match
                     for n in xrange(regcount):
-                        if category and reTypes[n] == 'C':
-                            found = re.search(regexes[n], category)
-                            if not found:
-                                logging.debug("Filter rejected on rule %d", n)
-                                result = False
-                                break
-                        else:
-                            found = re.search(regexes[n], title)
-                            if reTypes[n] == 'M' and not found:
-                                logging.debug("Filter rejected on rule %d", n)
-                                result = False
-                                break
-                            if found and reTypes[n] == 'A':
-                                logging.debug("Filter matched on rule %d", n)
-                                result = True
-                                break
-                            if found and reTypes[n] == 'R':
-                                logging.debug("Filter rejected on rule %d", n)
-                                result = False
-                                break
+                        if reEnabled[n]:
+                            if category and reTypes[n] == 'C':
+                                found = re.search(regexes[n], category)
+                                if not found:
+                                    logging.debug("Filter rejected on rule %d", n)
+                                    result = False
+                                    break
+                            else:
+                                if regexes[n]:
+                                    found = re.search(regexes[n], title)
+                                else:
+                                    found = False
+                                if reTypes[n] == 'M' and not found:
+                                    logging.debug("Filter rejected on rule %d", n)
+                                    result = False
+                                    break
+                                if found and reTypes[n] == 'A':
+                                    logging.debug("Filter matched on rule %d", n)
+                                    result = True
+                                    break
+                                if found and reTypes[n] == 'R':
+                                    logging.debug("Filter rejected on rule %d", n)
+                                    result = False
+                                    break
 
                     if len(reCats):
                         if notdefault(reCats[n]):
