@@ -82,7 +82,7 @@ def init():
             continue
 
         if d.isdigit():
-            d = [int(d)]
+            d = [int(i) for i in d]
         else:
             d = range(1, 8)
 
@@ -158,6 +158,13 @@ def init():
         __SCHED.add_single_task(Bookmarks.do.run, 'Bookmarks', 20, kronos.method.sequential, None, None)
 
 
+    action, hour, minute = sabnzbd.bpsmeter.BPSMeter.do.get_quota()
+    if action:
+        logging.info('Setting schedule for quota check daily at %s:%s', hour, minute)
+        __SCHED.add_daytime_task(action, 'quota_reset', range(1, 8), None, (hour, minute),
+                                 kronos.method.sequential, [], None)
+
+
     # Subscribe to special schedule changes
     cfg.newzbin_bookmarks.callback(schedule_guard)
     cfg.bookmark_rate.callback(schedule_guard)
@@ -306,7 +313,11 @@ def analyse(was_paused=False):
         sabnzbd.downloader.Downloader.do.limit_speed(speedlimit)
     for serv in servers:
         try:
-            config.get_config('servers', serv).enable.set(servers[serv])
+            item = config.get_config('servers', serv)
+            value = servers[serv]
+            if bool(item.enable()) != bool(value):
+                item.enable.set(value)
+                sabnzbd.downloader.Downloader.do.init_server(serv, serv)
         except:
             pass
     config.save_config()
