@@ -1,6 +1,6 @@
 #!/usr/bin/python -OO
 # -*- coding: UTF-8 -*-
-# Copyright 2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,6 +28,8 @@ SKIN_TEXT = {
     'stage-filejoin'     : TT('Join files'), #: PP phase "filejoin"
     'stage-unpack'       : TT('Unpack'), #: PP phase "unpack"
     'stage-script'       : TT('Script'), #: PP phase "script"
+    'stage-source'       : TT('Source'), #: PP Source of the NZB (path or URL)
+    'stage-fail'         : TT('Failure'), #: PP Failure message
 
     'post-Completed'     : TT('Completed'), #: PP status
     'post-Failed'        : TT('Failed'), #: PP status
@@ -75,6 +77,7 @@ SKIN_TEXT = {
     'seconds' : TT('seconds'), #: Multiple seconds
     'day' : TT('day'),
     'days' : TT('days'),
+    'week' : TT('week'),
     'month' : TT('Month'),
     'year' : TT('Year'),
     'daily' : TT('Daily'),
@@ -90,6 +93,13 @@ SKIN_TEXT = {
     'thisMonth' : TT('This month'),
     'today' : TT('Today'),
     'total' : TT('Total'),
+    'on' : TT('on'),
+    'off' : TT('off'),
+    'parameters' : TT('Parameters'), #: Config: startup parameters of SABnzbd
+    'pythonVersion' : TT('Python Version'),
+    'homePage' : TT('Home page'), #: Home page of the SABnzbd project
+    'source' : TT('Source'), #: Where to find the SABnzbd sourcecode
+    'or' : TT('or'), #: Used in "IRC or IRC-Webaccess"
 
 # General template elements
     'signOn' : TT('The automatic usenet download tool'), #: SABnzbd's theme line
@@ -116,11 +126,12 @@ SKIN_TEXT = {
     'cmenu-servers' : TT('Servers'), #: Main menu item
     'cmenu-scheduling' : TT('Scheduling'), #: Main menu item
     'cmenu-rss' : TT('RSS'), #: Main menu item
-    'cmenu-emailN' : TT('Notifications'), #: Main menu item
+    'cmenu-notif' : TT('Notifications'), #: Main menu item
     'cmenu-email' : TT('Email'), #: Main menu item
     'cmenu-newzbin' : TT('Index Sites'), #: Main menu item
     'cmenu-cat' : TT('Categories'), #: Main menu item
     'cmenu-sorting' : TT('Sorting'), #: Main menu item
+    'cmenu-special' : TT('Special'), #: Main menu item
 
 # Footer
     'ft-download' : TT('Download Dir'), # Used in Footer
@@ -161,6 +172,7 @@ SKIN_TEXT = {
     'pr-high' : TT('High'),
     'pr-low' : TT('Low'),
     'pr-paused' : TT('Paused'),
+    'pr-stop' : TT('Stop'),
     'enterURL' : TT('Enter URL'),
     'enterID' : TT('&nbsp;or Report ID'),
 
@@ -197,7 +209,9 @@ SKIN_TEXT = {
     'removeNZB-Files' : TT('Remove NZB & Delete Files'), #: Queue page button
     'AofB' : TT('of'), #: Queue page, as in "4G *of* 10G"
     'missingArt': TT('Missing articles'), #: Caption for missing articles in Queue
-    'quotum-left' : TT('Quotum left'), #: Remaining quotum (displayed in Queue)
+    'quota-left' : TT('Quota left'), #: Remaining quota (displayed in Queue)
+    'manual' : TT('manual'), #: Manual reset of quota
+    'link-resetQuota' : TT('Reset Quota now'),
 
 # History page
     'purgeHist' : TT('Purge History'), #: History page button
@@ -306,6 +320,8 @@ SKIN_TEXT = {
     'opt-disableApikey' : TT('Disable API-key'),
     'explain-disableApikey' : TT('Do not require the API key.'),
     'explain-disableApikeyWarn' : TT('USE AT YOUR OWN RISK!'),
+    'qr-code' : TT('QR Code'), #: Button to show QR code of APIKEY
+    'explain-qr-code' : TT('API Key QR Code'), #: Explanation for QR code of APIKEY
 
 # Config->Folders
     'folderConfig' : TT('Folder configuration'),
@@ -399,6 +415,7 @@ SKIN_TEXT = {
     'explain-auto_sort' : TT('Automatically sort items by (average) age.'),
     'opt-check_new_rel' : TT('Check for New Release'),
     'explain-check_new_rel' : TT('Weekly check for new SABnzbd release.'),
+    'also-test' : TT('Also test releases'), #: Pick list for weekly test for new releases
     'opt-replace_spaces' : TT('Replace Spaces in Foldername'),
     'explain-replace_spaces' : TT('Replace spaces with underscores in folder names.'),
     'opt-replace_dots' : TT('Replace dots in Foldername'),
@@ -423,37 +440,42 @@ SKIN_TEXT = {
     'swtag-queue' : TT('Queue'),
     'swtag-pp' : TT('Post processing'),
     'swtag-naming' : TT('Naming'),
-    'swtag-quotum' : TT('Quotum'),
-    'opt-quotum_size' : TT('Size'), #: Size of the download quotum
-    'explain-quotum_size' : TT('How much can be downloaded this month (K/M/G)'),
-    'opt-quotum_day' : TT('Reset day'), #: Reset day of the download quotum
-    'explain-quotum_day' : TT('On which day of the month or week (1=Monday) does your ISP reset the quotum? (Optionally with hh:mm)'),
-    'opt-quotum_resume' : TT('Auto resume'), #: Auto-resume download on the reset day
-    'explain-quotum_resume' : TT('Should downloading resume after the quotum is reset?'),
-    'opt-quotum_period' : TT('Quotum period'), #: Does the quotum get reset every day, week or month?
-    'explain-quotum_period' : TT('Does the quotum get reset each day, week or month?'),
+    'swtag-quota' : TT('Quota'),
+    'opt-quota_size' : TT('Size'), #: Size of the download quota
+    'explain-quota_size' : TT('How much can be downloaded this month (K/M/G)'),
+    'opt-quota_day' : TT('Reset day'), #: Reset day of the download quota
+    'explain-quota_day' : TT('On which day of the month or week (1=Monday) does your ISP reset the quota? (Optionally with hh:mm)'),
+    'opt-quota_resume' : TT('Auto resume'), #: Auto-resume download on the reset day
+    'explain-quota_resume' : TT('Should downloading resume after the quota is reset?'),
+    'opt-quota_period' : TT('Quota period'), #: Does the quota get reset every day, week or month?
+    'explain-quota_period' : TT('Does the quota get reset each day, week or month?'),
     'opt-pre_check' : TT('Check before download'),
     'explain-pre_check' : TT('Try to predict successful completion before actual download (slower!)'),
+    'opt-max_art_tries' : TT('Maximum retries'),
+    'explain-max_art_tries' : TT('Maximum number of retries per server'),
+    'opt-max_art_opt' : TT('Only for optional servers'),
+    'explain-max_art_opt' : TT('Apply maximum retries only to optional servers'),
 
 
 # Config->Server
-    'configServer' : TT('Server configuration'),
-    'defServer' : TT('Server definition'),
-    'addServer' : TT('Add Server'),
-    'srv-host' : TT('Host'),
-    'srv-port' : TT('Port'),
-    'srv-username' : TT('Username'),
-    'srv-password' : TT('Password'),
-    'srv-timeout' : TT('Timeout'),
-    'srv-connections' : TT('Connections'),
-    'srv-retention' : TT('Retention time'),
-    'srv-ssl' : TT('SSL'),
-    'srv-fillserver' : TT('Backup server'),
-    'srv-optional' : TT('Optional'),
-    'srv-enable' : TT('Enable'),
-    'button-addServer' : TT('Add Server'),
-    'button-delServer' : TT('Remove Server'),
-    'button-testServer' : TT('Test Server'),
+    'configServer' : TT('Server configuration'), #: Caption
+    'defServer' : TT('Server definition'), # Caption
+    'addServer' : TT('Add Server'), #: Caption
+    'srv-host' : TT('Host'), #: Server hostname or IP
+    'srv-port' : TT('Port'), #: Server port
+    'srv-username' : TT('Username'), #: Server username
+    'srv-password' : TT('Password'), #: Server password
+    'srv-timeout' : TT('Timeout'), #: Server timeout
+    'srv-connections' : TT('Connections'), #: Server: amount of connections
+    'srv-retention' : TT('Retention time'), #: Server's retention time in days
+    'srv-ssl' : TT('SSL'), #: Server SSL tickbox
+    'srv-fillserver' : TT('Backup server'), #: Backup server tickbox
+    'srv-optional' : TT('Optional'), #: Server optional tickbox
+    'srv-enable' : TT('Enable'), #: Enable server tickbox
+    'button-addServer' : TT('Add Server'), #: Button: Add server
+    'button-delServer' : TT('Remove Server'), #: Button: Remove server
+    'button-testServer' : TT('Test Server'), #: Button: Test server
+    'button-clrServer' : TT('Clear Counters'), #: Button: Clear server's byte counters
     'srv-testing' : TT('Testing server details...'),
     'srv-testHint' : TT('Click below to test.'),
     'srv-bandwidth' : TT('Bandwidth'),
@@ -499,8 +521,7 @@ SKIN_TEXT = {
     'feedSettings' : TT('Settings'), #: Tab title for Config->Feeds
     'filters' : TT('Filters'), #: Tab title for Config->Feeds
 
-# Config->Email
-    'configEmail' : TT('Email Notifications'), #: Main Config page
+    'configEmail' : TT('Notifications'), #: Main Config page
     'emailOptions' : TT('Email Options'), #: Section header
     'opt-email_endjob' : TT('Email Notification On Job Completion'),
     'email-never' : TT('Never'), #: When to send email
@@ -623,6 +644,15 @@ SKIN_TEXT = {
     'button-ShowNameF' : TT('Show Name folder'),
     'button-YMF' : TT('Year-Month Folders'),
     'button-DailyF' : TT('Daily Folders'),
+    'case-adjusted' : TT('case-adjusted'), #: Note for title expression in Sorting that does case adjustment
+    'sortResult' : TT('Processed Result'),
+
+# Config->Special
+    'explain-special' : TT('Rarely used options. For their meaning and explanation, click on the Help button to go to the Wiki page.<br>'
+                           'Don\'t change these without checking the Wiki first, as some have serious side-effects.<br>'
+                           'The default values are between parentheses.'),
+    'sptag-boolean' : TT('Switches'),
+    'sptag-entries' : TT('Values'),
 
 # NZO
     'nzoDetails' : TT('Edit NZB Details'), #: Job details page
@@ -829,6 +859,8 @@ SKIN_TEXT = {
     'wizard-step-four' :  TT('Step Four'), #: Wizard step
     'wizard-step-five' :  TT('Step Five'), #: Wizard step
     'wizard-port-eg' : TT('E.g. 119 or 563 for SSL'), #: Wizard port number examples
+    'wizard-exit' : TT('Exit SABnzbd'), #: Wizard EXIT button on first page
+    'wizard-start' : TT('Start Wizard'), #: Wizard START button on first page
 
 #Special
     'yourRights' : TT('''
