@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -40,8 +40,9 @@ import sabnzbd.dirscanner as dirscanner
 from sabnzbd.nzbqueue import NzbQueue
 import sabnzbd.cfg as cfg
 
+_BAD_GZ_HOSTS = ('.zip', 'nzbsa.co.za', 'newshost.za.net')
+
 #------------------------------------------------------------------------------
-_RETRIES = 10
 
 class URLGrabber(Thread):
     do = None  # Link to instance of the thread
@@ -51,25 +52,26 @@ class URLGrabber(Thread):
         self.queue = Queue.Queue()
         for tup in NzbQueue.do.get_urls():
             url, nzo = tup
-            self.queue.put((url, nzo, _RETRIES))
+            self.queue.put((url, nzo))
         self.shutdown = False
-        self.matrix_wait = 0
         URLGrabber.do = self
 
-    def add(self, url, future_nzo):
-        """ Add an URL to the URLGrabber queue """
-        self.queue.put((url, future_nzo, _RETRIES))
+    def add(self, url, future_nzo, when=None):
+        """ Add an URL to the URLGrabber queue, 'when' is seconds from now """
+        if when and future_nzo:
+            future_nzo.wait = time.time() + when
+        self.queue.put((url, future_nzo))
 
     def rm_bookmark(self, url):
         """ Add removal request for nzbmatrix bookmark """
         if 'nzbmatrix.com' in url and cfg.matrix_del_bookmark():
             url = url.replace('download.php?', 'bookmarks.php?action=remove&')
-            self.queue.put((url, None, _RETRIES))
+            self.add(url, None)
 
     def stop(self):
         logging.info('URLGrabber shutting down')
         self.shutdown = True
-        self.queue.put((None, None, 0))
+        self.add(None, None)
 
     def run(self):
         logging.info('URLGrabber starting up')
@@ -79,8 +81,15 @@ class URLGrabber(Thread):
             # Don't pound the website!
             time.sleep(5.0)
 
-            (url, future_nzo, retry_count) = self.queue.get()
+            (url, future_nzo) = self.queue.get()
+
             if not url:
+                # stop signal, go test self.shutdown
+                continue
+            if future_nzo and future_nzo.wait and future_nzo.wait > time.time():
+                # Requeue when too early and still active
+
+                self.add(url, future_nzo)
                 continue
             url = url.replace(' ', '')
 
@@ -90,7 +99,7 @@ class URLGrabber(Thread):
                     # If nzo entry deleted, give up
                     try:
                         deleted = future_nzo.deleted
-                    except:
+                    except AttributeError:
                         deleted = True
                     if deleted:
                         logging.debug('Dropping URL %s, job entry missing', url)
@@ -99,22 +108,20 @@ class URLGrabber(Thread):
                 # Add nzbmatrix credentials if needed
                 url, matrix_id = _matrix_url(url)
 
-                # When still waiting for nzbmatrix wait period, requeue
-                if matrix_id and self.matrix_wait > time.time():
-                    self.queue.put((url, future_nzo, retry_count))
-                    continue
-
                 # _grab_url cannot reside in a function, because the tempfile
                 # would not survive the end of the function
                 if del_bookmark:
                     logging.info('Removing nzbmatrix bookmark %s', matrix_id)
                 else:
                     logging.info('Grabbing URL %s', url)
-                opener = urllib.FancyURLopener({})
+                if '.nzbsrus.' in url:
+                    opener = urllib.URLopener({})
+                else:
+                    opener = urllib.FancyURLopener({})
                 opener.prompt_user_passwd = None
                 opener.addheaders = []
                 opener.addheader('User-Agent', 'SABnzbd+/%s' % sabnzbd.version.__version__)
-                if '.zip' not in url:
+                if not [True for item in _BAD_GZ_HOSTS if item in url]:
                     opener.addheader('Accept-encoding','gzip')
                 filename = None
                 category = None
@@ -150,28 +157,30 @@ class URLGrabber(Thread):
 
                 if matrix_id:
                     fn, msg, retry, wait = _analyse_matrix(fn, matrix_id)
-                    if retry and wait > 0:
-                        self.matrix_wait = time.time() + wait
-                        logging.debug('Retry URL %s after waiting', url)
-                        self.queue.put((url, future_nzo, retry_count))
+                    if not fn:
+                        if retry:
+                            logging.info(msg)
+                            logging.debug('Retry nzbmatrix item %s after waiting %s sec', matrix_id, wait)
+                            self.add(url, future_nzo, wait)
+                        else:
+                            logging.error(msg)
+                            misc.bad_fetch(future_nzo, clean_matrix_url(url), msg, retry=True)
                         continue
                     category = _MATRIX_MAP.get(category, category)
+
+                    if del_bookmark:
+                        # No retries of nzbmatrix bookmark removals
+                        continue
+
                 else:
-                    msg = ''
-                    retry = True
-
-                # Check if the filepath is specified, if not, check if a retry is allowed.
-                if not fn:
-                    retry_count -= 1
-                    if retry_count > 0 and retry:
-                        logging.info('Retry URL %s', url)
-                        self.queue.put((url, future_nzo, retry_count))
-                    elif not del_bookmark:
-                        misc.bad_fetch(future_nzo, url, msg, retry=True)
-                    continue
-
-                if del_bookmark:
-                    continue
+                    fn, msg, retry, wait = _analyse_others(fn, url)
+                    if not fn:
+                        if retry:
+                            logging.info('Retry URL %s', url)
+                            self.add(url, future_nzo, wait)
+                        else:
+                            misc.bad_fetch(future_nzo, url, msg, retry=True)
+                        continue
 
                 if not filename:
                     filename = os.path.basename(url) + '.nzb'
@@ -190,22 +199,21 @@ class URLGrabber(Thread):
 
                 # Check if nzb file
                 if os.path.splitext(filename)[1].lower() in ('.nzb', '.gz'):
-                    res = dirscanner.ProcessSingleFile(filename, fn, pp=pp, script=script, cat=cat, priority=priority, \
+                    res, nzo_ids = dirscanner.ProcessSingleFile(filename, fn, pp=pp, script=script, cat=cat, priority=priority, \
                                                        nzbname=nzbname, nzo_info=nzo_info, url=future_nzo.url)
                     if res == 0:
                         NzbQueue.do.remove(future_nzo.nzo_id, add_to_history=False)
-                    elif res == -2:
-                        retry_count -= 1
-                        if retry_count > 0:
-                            logging.info('Incomplete NZB, retry %s', url)
-                            self.queue.put((url, future_nzo, retry_count))
-                        else:
-                            misc.bad_fetch(future_nzo, url, retry=True, content=True)
                     else:
-                        misc.bad_fetch(future_nzo, url, retry=True, content=True)
+                        if res == -2:
+                            logging.info('Incomplete NZB, retry after 5 min %s', url)
+                            when = 300
+                        else:
+                            logging.info('Unknown error fetching NZB, retry after 2 min %s', url)
+                            when = 120
+                        self.add(url, future_nzo, when)
                 # Check if a supported archive
                 else:
-                    if dirscanner.ProcessArchiveFile(filename, fn, pp, script, cat, priority=priority, url=future_nzo.url) == 0:
+                    if dirscanner.ProcessArchiveFile(filename, fn, pp, script, cat, priority=priority, url=future_nzo.url)[0] == 0:
                         NzbQueue.do.remove(future_nzo.nzo_id, add_to_history=False)
                     else:
                         # Not a supported filetype, not an nzb (text/html ect)
@@ -213,7 +221,8 @@ class URLGrabber(Thread):
                             os.remove(fn)
                         except:
                             pass
-                        misc.bad_fetch(future_nzo, url, retry=True, content=True)
+                        logging.info('Unknown filetype when fetching NZB, retry after 30s %s', url)
+                        self.add(url, future_nzo, 30)
             except:
                 logging.error('URLGRABBER CRASHED', exc_info=True)
                 logging.debug("URLGRABBER Traceback: ", exc_info=True)
@@ -222,7 +231,8 @@ class URLGrabber(Thread):
 
 
 #-------------------------------------------------------------------------------
-_RE_NZBMATRIX = re.compile(r'nzbmatrix.com/(.*)[\?&]id=(\d+)', re.I)
+_RE_NZBMATRIX = re.compile(r'nzbmatrix\.com/(.*)[\?&]id=(\d+)', re.I)
+_RE_NZBXXX    = re.compile(r'nzbxxx\.com/(.*)[\?&]id=(\d+)', re.I)
 _RE_NZBMATRIX_USER = re.compile(r'&username=([^&=]+)', re.I)
 _RE_NZBMATRIX_API  = re.compile(r'&apikey=([^&=]+)', re.I)
 
@@ -231,16 +241,41 @@ def _matrix_url(url):
 
     matrix_id = 0
     m = _RE_NZBMATRIX.search(url)
+    if not m:
+        mx = _RE_NZBXXX.search(url)
+
+    if m:
+        site = 'nzbmatrix.com'
+        user = urllib.quote_plus(cfg.matrix_username())
+        key = urllib.quote_plus(cfg.matrix_apikey())
+    elif mx:
+        site = 'nzbxxx.com'
+        user = urllib.quote_plus(cfg.xxx_username())
+        key = urllib.quote_plus(cfg.xxx_apikey())
+        m = mx
+
     if m:
         matrix_id = m.group(2)
         if not _RE_NZBMATRIX_USER.search(url) or not _RE_NZBMATRIX_API.search(url):
-            user = urllib.quote_plus(cfg.matrix_username())
-            key = urllib.quote_plus(cfg.matrix_apikey())
-            url = '%s://api.nzbmatrix.com/v1.1/download.php?id=%s&username=%s&apikey=%s' % \
-                  (_PROTOCOL, matrix_id, user, key)
+            url = '%s://api.%s/v1.1/download.php?id=%s&username=%s&apikey=%s' % \
+                  (_PROTOCOL, site, matrix_id, user, key)
     return url, matrix_id
 
 
+def clean_matrix_url(url):
+    ''' Return nzbmatrix url without user credentials '''
+    site = 'nzbmatrix.com'
+    m = _RE_NZBMATRIX.search(url)
+    if not m:
+        m = _RE_NZBXXX.search(url)
+        site = 'nzbxxx.com'
+
+    if m:
+        matrix_id = m.group(2)
+        url = '%s://api.%s/v1.1/download.php?id=%s' % (_PROTOCOL, site, matrix_id)
+    return url
+
+
 _RE_MATRIX_ERR = re.compile(r'please_wait[_ ]+(\d+)', re.I)
 
 def _analyse_matrix(fn, matrix_id):
@@ -248,45 +283,100 @@ def _analyse_matrix(fn, matrix_id):
         returns fn|None, error-message|None, retry, wait-seconds
     """
     msg = ''
+    wait = 0
     if not fn:
-        # No response, just retry
-        return (None, msg, True, 0)
+        logging.debug('No response from nzbmatrix, retry after 60 sec')
+        return None, msg, True, 60
     try:
         f = open(fn, 'r')
-        data = f.read(40)
+        data = f.read(40).lower()
         f.close()
     except:
-        return (None, msg, True, 0)
+        logging.debug('Problem with tempfile %s from nzbmatrix, retry after 60 sec', fn)
+        return None, msg, True, 60
 
     # Check for an error response
-    if data and data.startswith('error'):
-        wait = 0
-        # Check for daily limit
-        if 'daily_limit' in data or 'limit is reached' in data:
+    if data and '<!DOCTYPE' in data:
+        # We got HTML, probably a temporary problem, keep trying
+        msg = Ta('Invalid nzbmatrix report number %s') % matrix_id
+        wait = 300
+    elif data and data.startswith('error'):
+        txt = misc.match_str(data, ('invalid_login', 'invalid_api', 'disabled_account', 'vip_only'))
+        if txt:
+            if 'vip' in txt:
+                msg = Ta('You need an nzbmatrix VIP account to use the API')
+            else:
+                msg = (Ta('Invalid nzbmatrix credentials') + ' (%s)') % txt
+            return None, msg, False, 0
+        elif 'limit_reached' in data:
+            msg = 'Too many nzbmatrix hits, waiting 10 min'
+            wait = 600
+        elif misc.match_str(data, ('daily_limit', 'limit is reached')):
             # Daily limit reached, just wait an hour before trying again
+            msg = 'Daily limit nzbmatrix reached, waiting 1 hour'
             wait = 3600
+        elif 'no_nzb_found' in data:
+            msg = Ta('Invalid nzbmatrix report number %s') % matrix_id
+            wait = 300
         else:
             # Check if we are required to wait - if so sleep the urlgrabber
             m = _RE_MATRIX_ERR.search(data)
             if m:
                 wait = min(int(m.group(1)), 600)
             else:
-                # Clear error message, don't retry
                 msg = Ta('Problem accessing nzbmatrix server (%s)') % data
-                return (None, msg, False, 0)
-        if wait:
-            logging.debug('Further NzbMatrix grabs after %s sec', wait)
-            # Return, but tell the urlgrabber to retry
-            return (None, msg, True, wait)
-
-    if data.startswith("<!DOCTYPE"):
-        # We got HTML, probably a temporary problem, keep trying
-        msg = Ta('Invalid nzbmatrix report number %s') % matrix_id
-        return (None, msg, True, 0)
+                wait = 60
+    if wait:
+        # Return, but tell the urlgrabber to retry
+        return None, msg, True, wait
 
     return fn, msg, False, 0
 
 
+
+RUS_FATAL = ('DENIED_MISSING_CREDENTIALS', 'DENIED_NO_ACCOUNT',
+             'DENIED_INVALID_CREDENTIALS', 'INCORRECT_URL',
+             'NZB_DELETED', 'POST_NUKED', 'FILE_UNAVAILABLE'
+            )
+RUS_15M =   ('SQL_ERROR', 'SERVICE_OFFLINE')
+RUS_60M =   ('MAX_DOWNLOAD_REACHED_UPGRADE_TO_VIP', 'MAX_DOWNLOAD_REACHED')
+
+def _analyse_others(fn, url):
+    """ Analyse respons of indexer
+        returns fn|None, error-message|None, retry, wait-seconds
+    """
+    msg = ''
+    wait = 0
+    if not fn:
+        logging.debug('No response from indexer, retry after 60 sec')
+        return None, msg, True, 60
+    try:
+        f = open(fn, 'r')
+        data = f.read(100)
+        f.close()
+    except:
+        logging.debug('Problem with tempfile %s from indexer, retry after 60 sec', fn)
+        return None, msg, True, 60
+
+    # Check for an error response
+    if not data:
+        logging.debug('Received nothing from indexer, retry after 60 sec')
+        return None, msg, True, 60
+
+    if '.nzbsrus.' in url:
+        # Partial support for nzbsrus.com's API
+        if misc.match_str(data, RUS_FATAL):
+            logging.debug('nzbsrus says: %s, abort', data)
+            return None, data, False, 0
+        if misc.match_str(data, RUS_15M):
+            logging.debug('nzbsrus says: %s, wait 15m', data)
+            return None, data, True, 900
+        if misc.match_str(data, RUS_60M):
+            logging.debug('nzbsrus says: %s, wait 60m', data)
+            return None, data, True, 3600
+
+    return fn, msg, False, 0
+
 #------------------------------------------------------------------------------
 _MATRIX_MAP = {
 '28' : 'anime.all',
@@ -294,16 +384,12 @@ _MATRIX_MAP = {
 '19' : 'apps.mac',
 '21' : 'apps.other',
 '18' : 'apps.pc',
-'55' : 'apps.phone',
 '52' : 'apps.portable',
 '53' : 'documentaries.hd',
 '9'  : 'documentaries.std',
-'16' : 'games.dreamcast',
 '45' : 'games.ds',
-'46' : 'games.gamecube',
 '17' : 'games.other',
 '10' : 'games.pc',
-'15' : 'games.ps1',
 '11' : 'games.ps2',
 '43' : 'games.ps3',
 '12' : 'games.psp',
@@ -318,26 +404,25 @@ _MATRIX_MAP = {
 '50' : 'movies.hd (image)',
 '42' : 'movies.hd (x264)',
 '4'  : 'movies.other',
-'3'  : 'movies.svcd/vcd',
-'48' : 'movies.wmv-hd',
 '24' : 'music.dvd',
 '23' : 'music.lossless',
 '22' : 'music.mp3, albums',
 '47' : 'music.mp3, singles',
 '27' : 'music.other',
 '25' : 'music.video',
+'55' : 'other.android',
 '49' : 'other.audio, books',
 '36' : 'other.e-books',
-'33' : 'other.emulation',
 '39' : 'other.extra, pars/fills',
 '37' : 'other.images',
-'38' : 'other.mobile, phone',
+'38' : 'other.iOS/iPhone',
 '40' : 'other.other',
-'34' : 'other.ppc/pda',
 '26' : 'other.radio',
-'6'  : 'tv.divx/xvid',
-'5'  : 'tv.dvd',
-'41' : 'tv.hd',
+'5'  : 'tv.dvd (image)',
+'57' : 'tv.hd (image)',
+'41' : 'tv.hd (x264)',
 '8'  : 'tv.other',
+'6'  : 'tv.sd',
 '7'  : 'tv.sport/ent'
 }
+
