@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -105,7 +105,7 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
 
 
     if not username or not password:
-        nw.nntp.sock.sendall('ARTICLE test\r\n')
+        nw.nntp.sock.sendall('ARTICLE test@home\r\n')
         try:
             nw.lines = []
             nw.recv_chunk(block=True)
